# Journal.pm - The Object Class that provides the Investment Journal Object
#$Id: Journal.pm,v 1.7 2004/05/24 21:06:30 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Investments::Journal;

use strict;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);
use HTMLObject::Normal;
use HTMLObject::FrameSet;
use Portal::Data::Variables;
use Portal::Data::Config;
use Portal::Accounting::Base;
use Portal::Accounting::Objects::AccountObject;
use Portal::Accounting::Objects::SubAccountObject;
use Portal::Accounting::DisplayItems;
use Portal::Accounting::DB::Main;
use Portal::Accounting::DB::Ledger;
use Portal::Accounting::Widgets;
use Portal::Accounting::Widgets::DateSelect;
use Portal::Accounting::DB::Security;
use Portal::Accounting::Objects::SecurityEventObject;
use Portal::Accounting::Data::Config;
use Portal::Accounting::Data::Variables;

require Exporter;

@ISA = qw(Portal::Accounting::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '$Revision: 1.7 $';

=head1 NAME

Journal - Object used to build a Journal Class.

=head1 SYNOPSIS

  use Portal::Accounting::Investments::Journal;
  my $obj = Portal::Accounting::Investments::Journal->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

Journal is the main class for the Journal.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(name, code, ledgerCode)

 Creates a new instance of the Portal::Accounting::Investments::Journal module.
 See Portal::Accounting::Base(3) for a listing of required arguments.

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( @_ );

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module
  $self->{state} = "Investments::Journal";
  $self->{commands} = { display => "Investment Journal",
              displayEvents => "Security Transactions",
              inputFrame => "Input New Transaction",
              blank => "blank",
              lotSelect => "Select Lot",
              sharesSelect => "Select Shares from Lots",
              insert => "Insert Transaction",
              inputScreen => "Input New Transaction",
              deleteEntry => "Delete Transaction"};

  # do validation
  if (!$self->isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

sub run
{
  my $self = shift;
  if (!defined $self->input->{'ledger'})
  {
    $self->prefixError( "Error.  Must specify a ledger in Portal::Accounting::Investments::Journal->$self->{input}->{command}.");
    return undef;
  }
  if (! defined $self->input->{'ledgerCode'})
  {
    $self->prefixError( "Error.  Must specify a ledger code in Portal::Accounting::Investments::Journal->$self->{input}->{command}.");
    return undef;
  }
  $self->{formInfo} .= "<input type=\"hidden\" name=\"ledger\" value=\"$self->{input}->{ledger}\">\n";
  $self->{formInfo} .= "<input type=\"hidden\" name=\"ledgerCode\" value=\"$self->{input}->{ledgerCode}\">\n";
  $self->{getInfo} .= "&ledger=".HTMLObject::Base::encodeString($self->{input}->{ledger})."&ledgerCode=".HTMLObject::Base::encodeString($self->{input}->{ledgerCode});
  return $self->SUPER::run(@_);
}

sub c_display
{
  my $doc = HTMLObject::Normal->new;
  my $ledger = $self->{input}->{ledger};
  my $ledgerCode = $self->{input}->{ledgerCode};;
  my $phrases = $self->{langObj};
  $doc->setTitle($phrases->{'Investments'});
  $doc->setFocus("body");
  $doc->print($self->menu."<hr>");
  $doc->print("Foo");
  return $doc;
}

sub menu
{
  my $menu = <<"END";
<a href="" title="$Refresh">$Refresh</a> |
END
  foreach my $action (sort keys %investmentActionHash)
  {
    $menu .= "<a href=\"$self->{getInfo}&command=$action\" title=\\\"$self->{variables}->{investmentActionHash}->{$action}\">$self->{variables}->{investmentActionHash}->{$action}</a> |\n";
  }
  $menu .= <<"END";
<a href="$self->{getInfo}&command=prices" title="$StockPrices">$StockPrices</a>
END
  return $menu;
}

sub display
{
  my $self = shift;
  my %args = ( errorStr => "", statusStr => "", accounts => "",
    stocks => "", actions => "", numEntriesToDisplay => 20,
    showEvents => "global+local", @_ );
  my $accounts = "";
  my $actiontype = "";
  my $actions = "";
  my $doc = HTMLObject::Normal->new;
  my $ledger = $self->{input}->{ledger};
  my $ledgerCode = $self->{input}->{ledgerCode};;
  my $phrases = $self->{langObj};
  my @selected;
  my $stocks = "";
  my $successString = "";  # used to pass in the success message to the create new event screen.
  my $dbObject = Portal::Accounting::DB::Main->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
  if ($dbObject->didErrorOccur)
  {
    $self->prefixError( $dbObject->errorMessage);
    return undef;
  }
  my $securityDBObject = $dbObject;

  if ($args{accounts} > 0 || $args{stocks} > 0 || $args{actions} > 0)
  {
    $accounts = $args{accounts};
    $stocks = $args{stocks};
    $actions = $args{actions};
  }
  else
  {
    if (exists $self->{input}->{accounts}) {
      $accounts = $self->{input}->{accounts}; }
    if (exists $self->{input}->{stocks}) {
      $stocks = $self->{input}->{stocks}; }
    if (exists $self->{input}->{actions}) {
      $actions = $self->{input}->{actions}; }
  }
  my $widget = Portal::Accounting::Widgets->new(langObj => $self->{langObj});  #!!! need to consolidate widgets under portal, xiwa or something else.
  if ($widget->didErrorOccur)
  {
    $self->prefixError( $widget->errorMessage());
    return undef;
  }
  my $display = Portal::Accounting::DisplayItems->new(langObj => $self->{langObj});#!!! need to consolidate widgets under portal, xiwa or something else.
  if ($display->didErrorOccur())
  {
    $self->prefixError($display->errorMessage());
    return undef;
  }

  my $currFiscalYearObj = $dbObject->getCurrentFiscalYear(ledgerCode => $ledgerCode);
  if (!defined $currFiscalYearObj)
  {
    $self->prefixError( $dbObject->errorMessage);
    return undef;
  }
  else
  {
    if ($currFiscalYearObj->didErrorOccur)
    {
      $self->prefixError( $currFiscalYearObj->errorMessage);
      return undef;
    }
  }
  # output the necessary JavaScript code.
  $doc->setFocus("javascript");
  $doc->print(<<"END_OF_CODE");

function selectAll(widget)
{
  var i;
  for (i=0; i < widget.length; i++)
  {
    widget.options[i].selected = true;
  }
}

function unselectAll(widget)
{
  var i;
  for (i=0; i < widget.length; i++)
  {
    widget.options[i].selected = false;
  }
}

END_OF_CODE

  $doc->setTitle("$ledger ". $phrases->map("Investment Journal"));
  $doc->setFocus("body");
  $doc->setLinkDecorations;

  my $Portfolio = $phrases->map("Portfolio");
  my $Refresh = $phrases->map("Refresh");
  my $ledgerURI = $doc->encodeString(string => $ledger);
  my $tmpStartDate = $currFiscalYearObj->{startDate};
  my $tmpEndDate = $currFiscalYearObj->{endDate};
  my $dateYear = $self->{methods}->getCurrentDate(format => "%Y");
  my $startDatePhrase = $self->{langObj}->map("startDate");
  my $endDatePhrase = $self->{langObj}->map("endDate");

  my %startDatePicker = $self->{methods}->generatePickerCode(type => "date", doc => $doc, form => "mainForm",
                          itemName => "startDate", itemValue => $tmpStartDate, year => $dateYear,
                          phrase => $startDatePhrase, class => "menu", linkClass => "menu", width => 300, height => 250);
  if ($self->{methods}->didErrorOccur)
  {
    $self->prefixError($self->{methods}->errorMessage);
    return undef;
  }
  my %endDatePicker = $self->{methods}->generatePickerCode(type => "date", doc => $doc, form => "mainForm",
                          itemName => "endDate", itemValue => $tmpEndDate, year => $dateYear,
                          phrase => $endDatePhrase, class => "menu", linkClass => "menu", width => 300, height => 250);
  if ($self->{methods}->didErrorOccur)
  {
    $self->prefixError($self->{methods}->errorMessage);
    return undef;
  }
  my @accountList = $dbObject->getListOfAccounts(location => "local", type => 3, ledgerCode => $ledgerCode);
  if ($dbObject->didErrorOccur)
  {
    $self->prefixError(errorString => $dbObject->errorMessage);
    return undef;
  }
  #grab list of stocks from database
  my @stocksList = $dbObject->getListOfSubAccounts(type => "7,70", orderBy => 'sub_acc_name');
  if ($dbObject->didErrorOccur)
  {
    $self->prefixError(errorString => $dbObject->errorMessage);
    return undef;
  }
  my $InvestmentWatchList = $doc->encodeString (string => "Investments::WatchList");
  my $WatchList = $phrases->map("Watch List");
  my $StockPrices = $phrases->map("Stock Prices");
  $doc->print(<<"END_OF_CODE");
<a href="" title="$Refresh">$Refresh</a> |
<a href="$self->{getInfo}&command=buy" title="$buy">$buy</a> |
<a href="$self->{getInfo}&command=sell" title="$sell">$sell</a> |
<a href="$self->{getInfo}&command=prices" title="$StockPrices">$StockPrices</a>
<hr>
<center><h1>Investment Journal</h1></center>
<form method="post" name="mainForm">
$self->{formInfo}
<input type="hidden" name="command" value="displayEvents">
<p><b>View Investment Journal Entries by</b> <font size="-1">(leave items blank or unselected to have it search over all entries for that item)</font>
<table>
  <tr>
    <td align="center" valign="top">
END_OF_CODE
  $doc->print(%startDatePicker);
  $doc->print(<<"END_OF_CODE");
    <br><input type="button" name="clearStartDate" value="Clear" onClick="document.mainForm.startDate.value=''">
    </td>
    <td align="center" valign="top">
END_OF_CODE
  $doc->print(%endDatePicker);
  $doc->print(<<"END_OF_CODE");
    <br><input type="button" name="clearEndDate" value="Clear" onClick="document.mainForm.endDate.value=''">
    </td>
    <td>&nbsp;</td>
  </tr>
  <tr>
    <td align="left" valign="top" colspan="2">
      <b>Show </b><select name="numEntriesToDisplay">
        <option value="10">10
        <option value="25">25
        <option value="50" selected>50
      </select>
      <b> entries at a time.</b>
      <input type="hidden" name="startIndex" value="1">
    </td>
    <td>
      <b>Show </b> <select name="showEvents">
        <option value="global">Global
        <option value="global+local" selected>All
        <option value="local">Local
      </select>
      <b>Events</b>
    </td>
  </tr>

  <tr>
    <td align="center"><b>Investment Accounts</b><br>
END_OF_CODE
  @selected = split /,/, $accounts;
  if (!$widget->printSelect(doc => $doc, name=> "accounts", size=>"5", data => \@accountList, multiple => "true", selected => \@selected))
  {
    $self->prefixError($widget->errorMessage());
    return undef;
  }
  $doc->setFocus("body");
  $doc->print(<<"END_OF_CODE");
      <br><input type="button" name="allAccounts" value="Select All" onClick="selectAll(document.mainForm.accounts)">
      <br><input type="button" name="noAccounts" value="UnSelect All" onClick="unselectAll(document.mainForm.accounts)">
    </td>
    <td align="center"><b>Stocks and Bonds</b><br>
END_OF_CODE
  @selected = split /,/, $stocks;
  if (!$widget->printSelect(doc => $doc, name=> "stocks", size=>"5", data => \@stocksList, multiple => "true", selected => \@selected))
  {
    $self->prefixError($widget->errorMessage());
    return undef;
  }
  $doc->setFocus("body");
  $doc->print(<<"END_OF_CODE");
      <br><input type="button" name="allStocks" value="Select All" onClick="selectAll(document.mainForm.stocks)">
      <br><input type="button" name="noStocks" value="UnSelect All" onClick="unselectAll(document.mainForm.stocks)">
    </td>
    <td align="center"><b>Actions</b><br>
      <select name="actions" multiple size="5">
END_OF_CODE
  my %actionhash = %{$self->{variables}->{investmentActionHash}};
  my %numberhash = %{$self->{variables}->{investmentNumberHash}};
  foreach my $hashIndex (sort keys %actionhash)
  {
    my $eval = $numberhash{$hashIndex};
    $doc->print("    <option value=\"$numberhash{$hashIndex}\">$actionhash{$hashIndex}\n");
  }
  $doc->print(<<"END_OF_CODE");
      </select>
        <br><input type="button" name="allActions" value="Select All" onClick="selectAll(document.mainForm.actions)">
        <br><input type="button" name="noActions" value="UnSelect All" onClick="unselectAll(document.mainForm.actions)">
      </td>
    </tr>
    <tr>
      <td align="left" colspan="3">
        <input type="Submit" value="Show Transactions">
      </td>
    </tr>
  </table>
  </p>
  </form>
END_OF_CODE

  return $doc;
}#/

sub blank
{
  return HTMLObject::Normal->new();
}

=item HTMLObject deleteEntry(refNum => , ledgerCode)

  This subroutine will delete an investment transaction from the database and output the next screen

=cut
sub deleteEntry
{
  my $self = shift;
  my %args =  ( @_ );
  my $refNum = $self->{input}->{refNum};
  if ($refNum !~ /^\d+$/)
  {
    $self->prefixError("refNum = '$refNum' is invalid!<br>\n");
    return undef;
  }
  my $securityDBObject = Portal::Accounting::DB::Security->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
  if ($securityDBObject->didErrorOccur)
  {
    $self->prefixError($securityDBObject->errorMessage);
    return undef;
  }
  my $result = $securityDBObject->deleteSecurityEvent(eventRefNum => $refNum);
  if ($securityDBObject->didErrorOccur)
  {
    $self->prefixError($securityDBObject->errorMessage);
    return undef;
  }
  my $doc = $self->displayEvents(dbObject => $securityDBObject);
  if ($self->didErrorOccur)
  {
    $self->prefixError($self->errorMessage);
    return undef;
  }

  my $code = "window.alert(\"Successfully deleted Investment Entry \'$refNum\'.\");";
  $doc->setOnload(code => $code);

  return $doc;
}

sub displayEvents
{
  my $self = shift;
  my %args =  ( @_ );
  my $doc = HTMLObject::Normal->new;
  if ($doc->didErrorOccur)
  {
    $self->prefixError($doc->errorMessage);
    return undef;
  }
  my $dbObject = Portal::Accounting::DB::Security->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
  if ($dbObject->didErrorOccur)
  {
    $self->prefixError($dbObject->errorMessage);
    return undef;
  }
  my $display = Portal::Accounting::DisplayItems->new(langObj => $self->{langObj});  #!!! replace with external widgets?
  if ($display->didErrorOccur)
  {
    $self->prefixError($display->errorMessage);
    return undef;
  }
  my $otherArgs = "";
  my $altArgs = "";
  my $siteDir = $self->{configObj}->{siteDir};
  my $ledgerCode;
  my %accounts;
  my %stocks;
  # validate input from $self->{input} and then gather the data and display it.
  my %args = (accounts => "", stocks => "", actions => "", numEntriesToDisplay => 20, showEvents => "global+local");
  $args{accounts} = $self->{input}->{accounts};
  $args{stocks} = $self->{input}->{stocks};
  $args{actions} = $self->{input}->{actions};
  $args{startDate} = $self->{input}->{startDate};
  $args{endDate} = $self->{input}->{endDate};
  $args{numEntriesToDisplay} = $self->{input}->{numEntriesToDisplay};
  $args{showEvents} = $self->{input}->{showEvents};
  $args{startIndex} = $self->{input}->{startIndex};
  $args{ledger} = $self->{input}->{ledger};
  $args{ledgerCode} = $self->{input}->{ledgerCode};
  $ledgerCode = $self->{input}->{ledgerCode};

  my $printable = (exists $self->{input}->{printable} ? "true" : "false");  # are we displaying for printing?
  $args{errorStr} = "";
  $self->{input}->{codeFontSize} = (exists $self->{input}->{codeFontSize} ? $self->{input}->{codeFontSize} : "-1");
  $self->{input}->{normalFontSize} = (exists $self->{input}->{normalFontSize} ? $self->{input}->{normalFontSize} : "+0");

  # do quick validation
  foreach my $arg (split /\0/, $args{accounts})
  {
    $otherArgs .= "&accounts=$arg";
    $altArgs .= (length $altArgs > 0 ? "        " : "") . "<input type=\"hidden\" name=\"accounts\" value=\"$arg\">\n";
  }
  $args{accounts} =~ s/\0/,/g;
  if (length $args{accounts} > 0 && $args{accounts} !~ /^\d+(,\d+)*$/)
  {
    $args{errorStr} .= "accounts = '$args{accounts}' is invalid!<br>\n";
    $args{accounts} = "";
    $doc = $self->display(%args);
    if ($self->didErrorOccur)
    {
      $self->prefixError($args{errStr});
      return undef;
    }
    return $doc;
  }
  foreach my $arg (split /\0/, $args{stocks})
  {
    $otherArgs .= "&stocks=$arg";
    $altArgs .= (length $altArgs > 0 ? "        " : "") . "<input type=\"hidden\" name=\"stocks\" value=\"$arg\">\n";
  }
  $args{stocks} =~ s/\0/,/g;
  if (length $args{stocks} > 0 && $args{stocks} !~ /^\d+(,\d+)*$/)
  {
    $args{errorStr} .= "stocks = '$args{stocks}' is invalid!<br>\n";
    $args{stocks} = "";
    $doc = $self->display(%args);
    if ($self->didErrorOccur)
    {
      $self->prefixError($args{errStr});
      return undef;
    }
    return $doc;
  }
  foreach my $arg (split /\0/, $args{actions})
  {
    $otherArgs .= "&actions=$arg";
    $altArgs .= (length $altArgs > 0 ? "        " : "") . "<input type=\"hidden\" name=\"actions\" value=\"$arg\">\n";
  }
  $args{actions} =~ s/\0/,/g;
  if (length $args{actions} > 0 && $args{actions} !~ /^(\d+|memo)(,(\d+|memo))*$/)
  {
    $args{errorStr} .= "actions = '$args{actions}' is invalid!<br>\n";
    $args{actions} = "";
    $doc = $self->display(%args);
    if ($self->didErrorOccur)
    {
      $self->prefixError($args{errStr});
      return undef;
    }
    return $doc;
  }
  $otherArgs .= "&numEntriesToDisplay=$args{numEntriesToDisplay}";
  $altArgs .= (length $altArgs > 0 ? "        " : "") . "<input type=\"hidden\" name=\"numEntriesToDisplay\" value=\"$args{numEntriesToDisplay}\">\n";
  if ($args{numEntriesToDisplay} < 5 || $args{numEntriesToDisplay} > 50 || $args{numEntriesToDisplay} !~ /^\d+$/)
  {
    $args{errorStr} .= "numEntriesToDisplay = '$args{numEntriesToDisplay}' is invalid!<br>\n";
    $args{numEntriesToDisplay} = 20;
    $doc = $self->display(%args);
    if ($self->didErrorOccur)
    {
      $self->prefixError($args{errStr});
      return undef;
    }
    return $doc;
  }
  $otherArgs .= "&showEvents=" . $doc->encodeString(string => $args{showEvents});
  $altArgs .= (length $altArgs > 0 ? "        " : "") . "<input type=\"hidden\" name=\"showEvents\" value=\"$args{showEvents}\">\n";
  if ($args{showEvents} !~ /^(global|global\+local|local)$/)
  {
    $args{errorStr} .= "showEvents = '$args{showEvents}' is invalid!<br>\n";
    $args{showEvents} = "global";
    $doc = $self->display(%args);
    if ($self->didErrorOccur)
    {
      $self->prefixError($args{errStr});
      return undef;
    }
    return $doc;
  }
  $otherArgs .= "&startDate=$args{startDate}&endDate=$args{endDate}";
  $altArgs .= (length $altArgs > 0 ? "        " : "") . "<input type=\"hidden\" name=\"startDate\" value=\"$args{startDate}\">\n        <input type=\"hidden\" name=\"endDate\" value=\"$args{endDate}\">\n";
  my $startDate = $args{startDate};
  my $endDate = $args{endDate};
  if (length $startDate > 0)
  {
    my @dateResult = $self->{methods}->fixupDate(date => $startDate);
    if (!$dateResult[0])
    {
      $args{errorStr} .= "startDate = '$startDate' is invalid!<br>\n";
      $args{startDate} = "";
      $doc = $self->display(%args);
      if ($self->didErrorOccur)
      {
        $self->prefixError();
        return undef;
      }
      return $doc;
    }
    else
    {
      # make sure the date is "fixed".
      $startDate = $dateResult[1];
    }
  }
  if (length $endDate > 0)
  {
    my @dateResult = $self->{methods}->fixupDate(date => $endDate);
    if (!$dateResult[0])
    {
      $args{errorStr} .= "endDate = '$endDate' is invalid!<br>\n";
      $args{endDate} = "";
      $doc = $self->display(%args);
      if ($self->didErrorOccur)
      {
        $self->prefixError();
        return undef;
      }
      return $doc;
    }
    else
    {
      # make sure the date is "fixed".
      $endDate = $dateResult[1];
    }
  }
  #$otherArgs .= "&startIndex=$args{startIndex}";
  if ($args{startIndex} !~ /^\d+$/ || $args{startIndex} < 1)
  {
    $args{errorStr} .= "startIndex = '$args{startIndex}' is invalid!<br>\n";
    $args{startIndex} = 1;
    $doc = $self->display(%args);
    if ($self->didErrorOccur)
    {
      $self->prefixError;
      return undef;
    }
    return $doc;
  }

  # gather the data
  my $securityObj = Portal::Accounting::DB::Security->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
  if ($securityObj->didErrorOccur)
  {
    $self->prefixError($securityObj->errorMessage);
    return undef;
  }

  my $investmentJournalViewObj = $securityObj->getSecurityEventsForDisplay(%args);
  if ($securityObj->didErrorOccur)
  {
    $self->prefixError( $securityObj->errorMessage);
    return undef;
  }

  if (!$investmentJournalViewObj->isValid)
  {
    $self->prefixError( $investmentJournalViewObj->errorMessage);
    return undef;
  }

  $doc->setTitle("Investment Journal");
  $doc->setFocus("body");
  $doc->setLinkDecorations;
  my $ledgerURI = $doc->encodeString(string => $args{ledger});
  my $codeFontSize = $self->{input}->{codeFontSize};
  my $codeFontSizeURI = $doc->encodeString(string => $codeFontSize);
  my $normalFontSize = $self->{input}->{normalFontSize};
  my $normalFontSizeURI = $doc->encodeString(string => $normalFontSize);
  if ($printable eq "false")
  {
    $doc->setFocus("javascript");
    $doc->print(<<"END_OF_CODE");

function confirmDelete(refNum)
{
  return window.confirm("Do you want to Delete Investment Event with RefNum = '"+refNum+"'?\\nPress OK to Delete, Cancel otherwise.");
}

function changeFont()
{
  document.fontForm.submit();
}

END_OF_CODE

    $doc->setFocus("body");
    $doc->print("<center><h3>Investment Journal</h3></center>\n");

    my $startIndex = $args{startIndex};
    $doc->print(<<"END_OF_CODE");
<table border="0" width="100%" cellpadding="0" cellspacing="0">
  <tr>
    <td align="left"><a href="$self->{getInfo}&command=display" onMouseOver="window.status='Investment Journal'"><font size="-1">Investment Journal</font></a>&nbsp;|
      <a href="$self->{getInfo}&command=displayEvents$otherArgs&startIndex=$startIndex&printable=true&codeFontSize=$codeFontSizeURI&normalFontSize=$normalFontSizeURI" onMouseOver="window.status='Print Page'"><font size="-1">Print Page</font></a>&nbsp;|
      <a href="$self->{getInfo}&command=inputScreen" onMouseOver="window.status='Create New Investment Transaction'"><font size="-1">New Transaction</font></a>
    </td>
    <td align="center">
      <form method="post" action="$siteDir/cgi-bin/index.cgi" name="fontForm">
        $self->{postInfo}
        <input type="hidden" name="command" value="displayEvents">
        <input type="hidden" name="startIndex" value="$startIndex">
        $altArgs
        <font size="-1">Change Font Sizes:<br>
        Code <select name="codeFontSize" onChange="changeFont(); return true;">
END_OF_CODE
    for (my $i=2; $i > -3; $i--)
    {
      my $selected = ($i > -1 ? ("+$i" eq $codeFontSize ? " selected" : "") : ("$i" eq $codeFontSize ? " selected" : ""));
      $doc->print("          <option value=\"" . ($i > -1 ? "+" : "") . "$i\"$selected>" . ($i > -1 ? "+" : "") . "$i\n");
    }
    $doc->print(<<"END_OF_CODE");
        </select>&nbsp;&nbsp;Normal <select name="normalFontSize" onChange="changeFont(); return true;">
END_OF_CODE
    for (my $i=2; $i > -3; $i--)
    {
      my $selected = ($i > -1 ? ("+$i" eq $normalFontSize ? " selected" : "") : ("$i" eq $normalFontSize ? " selected" : ""));
      $doc->print("          <option value=\"" . ($i > -1 ? "+" : "") . "$i\"$selected>" . ($i > -1 ? "+" : "") . "$i\n");
    }
    $doc->print(<<"END_OF_CODE");
        </select>
      </form>
    </td>
    <td align="right">
END_OF_CODE
    # now print the Prev, Curr, Next links and the select dropdown to jump around quicker with.
    my $length2 = $args{numEntriesToDisplay} * 2;
    if ($args{startIndex} > $args{numEntriesToDisplay}) # we need a previous link
    {
      my $displayString = "Prev (" . ($args{startIndex}-$args{numEntriesToDisplay}) . " - " . ($args{startIndex}-1) . ")";
      my $startIndex = $args{startIndex}-$args{numEntriesToDisplay};
      $doc->print(<<"END_OF_CODE");
      <a href="$self->{getInfo}&command=displayEvents$otherArgs&startIndex=$startIndex&codeFontSize=$codeFontSizeURI&normalFontSize=$normalFontSizeURI" onMouseOver="window.status='$displayString'"><font size="-1">$displayString</font></a>&nbsp;|&nbsp;
END_OF_CODE
    }

    # now print the current link
    my $displayString = "Curr (" . ($args{startIndex}) . " - " . ($investmentJournalViewObj->{totalEntries} > $args{startIndex}+$args{numEntriesToDisplay}-1 ? ($args{startIndex}+$args{numEntriesToDisplay}-1) : $investmentJournalViewObj->{totalEntries}) . ")";
    my $startIndex = $args{startIndex};
    if ($investmentJournalViewObj->{totalEntries} > 0)
    {
      $doc->print(<<"END_OF_CODE");
      <a href="$self->{getInfo}&command=displayEvents$otherArgs&startIndex=$startIndex&codeFontSize=$codeFontSizeURI&normalFontSize=$normalFontSizeURI" onMouseOver="window.status='$displayString'"><font size="-1">$displayString</font></a>
END_OF_CODE
    }

    if ($args{startIndex}+$args{numEntriesToDisplay} <= $investmentJournalViewObj->{totalEntries})  # we need the next link
    {
      my $displayString = "Next (" . ($args{startIndex}+$args{numEntriesToDisplay}) . " - " . ($investmentJournalViewObj->{totalEntries} > $args{startIndex}+$length2-1 ? ($args{startIndex}+$length2-1) : $investmentJournalViewObj->{totalEntries}) . ")";
      my $startIndex = $args{startIndex}+$args{numEntriesToDisplay};
      $doc->print(<<"END_OF_CODE");
      &nbsp;|&nbsp;<a href="$self->{getInfo}&command=displayEvents$otherArgs&startIndex=$startIndex&codeFontSize=$codeFontSizeURI&normalFontSize=$normalFontSizeURI" onMouseOver="window.status='$displayString'"><font size="-1">$displayString</font></a>
END_OF_CODE
    }
    my $header = ($investmentJournalViewObj->{totalEntries} > 0 ? "&nbsp;|&nbsp;&nbsp;" : "");
    $doc->print(<<"END_OF_CODE");
      $header<font size="-1"><b>$investmentJournalViewObj->{totalEntries}</b> Total, showing <b>$args{numEntriesToDisplay}</b>/page.</font>
    </td>
  </tr>
</table>
<br>
END_OF_CODE
  }
  else
  {
    # display the Investment Journal header and make it the link to return to the current page (non-printable) view.
    my $startIndex = $args{startIndex};
    $doc->print(<<"END_OF_CODE");
<center>
<a href="$self->{getInfo}&command=displayEvents$otherArgs&startIndex=$startIndex&codeFontSize=$codeFontSizeURI&normalFontSize=$normalFontSizeURI" onMouseOver="window.status='Return to Investment Journal View'"><font color="black"><h2>Investment Journal</h2></font></a>
</center>
<br>
END_OF_CODE
  }

  # lookup all stocks Numbers => Names and accounts Numbers => Names and store in hashes to pass into the securityEvent Object display method.
  my @list = $dbObject->getListOfAccounts(location => "local", type => 3, ledgerCode => $args{ledgerCode});
  if ($dbObject->didErrorOccur)
  {
    $self->prefixError(errorString => $dbObject->errorMessage);
    return undef;
  }
  # now make entries in the accounts hash for every account returned.
  foreach my $acctObj (@list)
  {
    $accounts{$acctObj->{number}} = $doc->formEncodeString(string => $acctObj->name);
  }

  my @list = $dbObject->getListOfSubAccounts(type => "70,7", orderBy => 'sub_acc_name');
  if ($dbObject->didErrorOccur)
  {
    $self->prefixError(errorString => $dbObject->errorMessage);
    return undef;
  }
  # now make entries in the stocks hash for every sub account returned.
  foreach my $stockObj (@list)
  {
    $stocks{$stockObj->{number}} = $doc->formEncodeString(string => $stockObj->name);
  }

  # get the start date of the current fiscal year and also find out if we are in the first fiscal year or not.
  my $currFiscalYearObj = $dbObject->getCurrentFiscalYear(ledgerCode => $args{ledgerCode});
  if ($dbObject->didErrorOccur)
  {
    $self->prefixError(errorString => $dbObject->errorMessage);
    return undef;
  }
  if (! defined $currFiscalYearObj)
  {
    $self->prefixError(errorString => "Current Fiscal Year info not returned!<br>\n");
    return undef;
  }
  my $fyStart = $currFiscalYearObj->startDate;

  my @knownFiscalYears = $dbObject->getAllKnownYears(ledgerCode => $args{ledgerCode});
  if ($dbObject->didErrorOccur)
  {
    $self->prefixError(errorString => $dbObject->errorMessage);
    return undef;
  }
  if (! defined @knownFiscalYears)
  {
    $self->prefixError(errorString => "All Known Fiscal Years not returned!<br>\n");
    return undef;
  }
  my $firstYearObj = $knownFiscalYears[0];
  my $inFirstYear = ($firstYearObj->year == $currFiscalYearObj->year ? 1 : 0);

  # now walk over the returned data and display the Investment Journal stuff.
  $doc->print(<<"END_OF_CODE");
<form method="post" action="$siteDir/cgi-bin/index.cgi" name="investViewForm">
  $self->{postInfo}
  <input type="hidden" name="command" value="deleteEntry">
  <input type="hidden" name="refNum" value="">
  <input type="hidden" name="startIndex" value="$args{startIndex}">
  $altArgs
  <input type="hidden" name="codeFontSize" value="$codeFontSize">
  <input type="hidden" name="normalFontSize" value="$normalFontSize">
<table border="0" cellpadding="0" cellspacing="0" width="100%">
END_OF_CODE

  # Sell grouping data variables.
  my $priceTotal = "0.00";
  my $sharesTotal = "0.00";
  my $commissionTotal = "0.00";
  my $masterEventRef = "-1";
  my $startColor = "lightgrey";
  my $numSells = 0;
  my $count = 0;
  my $lastColor = "white";

  for ($count=0; $count < scalar @{$investmentJournalViewObj->{securityEvents}}; $count++)
  {
    my $color = ($lastColor eq "white" ? "lightgrey" : "white");
    # see if we just closed out a sell run that needs to be totaled.
    if ($count > 0 && $investmentJournalViewObj->{securityEvents}[$count-1]->{eventCode} == 20 && $numSells > 1 && ($investmentJournalViewObj->{securityEvents}[$count]->{eventCode} ne "sell" || ($investmentJournalViewObj->{securityEvents}[$count]->{eventCode} == 20 && $investmentJournalViewObj->{securityEvents}[$count]->{masterEventRef} != $masterEventRef)))
    {
      $color = $startColor;
      $doc->print("  <tr bgcolor=\"$color\">\n");
      $doc->print("    <td width=\"" . ($printable eq "false" ? "95%" : "100%") . "\">\n");
      # output the totals.
      $doc->print("      <table border=\"1\" cellpadding=\"1\" cellspacing=\"0\" width=\"100%\">\n");
      $doc->print("        <tr>\n");
      $doc->print("          <td align=\"right\" width=\"44%\"><b>Totals:</b></td>\n");
      $doc->print("          <td align=\"right\" width=\"14%\"><i>" . $display->commify(value => sprintf("%.2f", $priceTotal)) . "</i></td>\n");
      $doc->print("          <td align=\"right\" width=\"14%\"><i>" . $display->commify(value => sprintf("%.2f", $sharesTotal)) . "</i></td>\n");
      $doc->print("          <td align=\"right\" width=\"15%\"><i>" . $display->commify(value => sprintf("%.2f", $commissionTotal)) . "</i></td>\n");
      $doc->print("          <td align=\"right\" width=\"13%\">&nbsp;</td>\n");
      $doc->print("        </tr>\n");
      $doc->print("      </table>\n");
      $doc->print("    </td>\n");
      if ($printable eq "false")
      {
        $doc->print("    <td width=\"5%\" align=\"center\">&nbsp;</td>\n");
      }
      $doc->print("  </tr>\n");

      # reset variables.
      $priceTotal = "0.00";
      $sharesTotal = "0.00";
      $commissionTotal = "0.00";
      $masterEventRef = "-1";
      $numSells = 0;
      $lastColor = $startColor;
    }
    $color = ($lastColor eq "white" ? "lightgrey" : "white");  # recalculate.
    if ($investmentJournalViewObj->{securityEvents}[$count]->{eventCode} == 20 && $numSells > 0 && $investmentJournalViewObj->{securityEvents}[$count]->{masterEventRef} == $masterEventRef)
    {
      $color = $startColor;
    }
    $doc->print("  <tr bgcolor=\"$color\">\n");
    $doc->print("    <td width=\"" . ($printable eq "false" ? "95%" : "100%") . "\">\n");
    my $output = $investmentJournalViewObj->{securityEvents}[$count]->display(printable => $printable,
                 stocks => \%stocks, accounts => \%accounts, displayObj => $display, codeFontSize => $codeFontSize, normalFontSize => $normalFontSize);
    if ($investmentJournalViewObj->{securityEvents}[$count]->didErrorOccur)
    {
      $args{errorStr} .=  $investmentJournalViewObj->{securityEvents}[$count]->errorMessage;
      $doc = $self->display(%args);
      if ($self->didErrorOccur)
      {
        $self->prefixError;
        return undef;
      }
      return $doc;
    }
    $output =~ s/^(.+)$/      $1/gm;
    $doc->print($output);
    $doc->print("    </td>\n");
    if ($printable eq "false")
    {
      $doc->print("    <td width=\"5%\" align=\"center\">\n");
      # check and see if they can delete the entry
      if ($self->transactionDeletable(calcDate => $investmentJournalViewObj->{securityEvents}[$count]->calcDate, fyStart => $fyStart, inFirstYear => $inFirstYear))
      {
        $doc->print("      <font size=\"-2\"><input type=\"button\" name=\"delete_$count\" value=\"D\" onclick=\"if (confirmDelete($investmentJournalViewObj->{securityEvents}[$count]->{eventRefNum})) { document.investViewForm.refNum.value=$investmentJournalViewObj->{securityEvents}[$count]->{eventRefNum}; document.investViewForm.submit(); }\"></font>\n");
      }
      else
      {
        $doc->print("      &nbsp;\n");
      }
      $doc->print("    </td>\n");
    }
    $doc->print("  </tr>\n");
    
    # see if we just started a sell run or if we are in one and need to update the values.
    if ($investmentJournalViewObj->{securityEvents}[$count]->{eventCode} == 20)
    {
      my $obj = $investmentJournalViewObj->{securityEvents}[$count];
      if ($numSells > 0 && $obj->{masterEventRef} == $masterEventRef)  # update
      {
        $numSells++;
        $priceTotal += $obj->{price};
        $sharesTotal += $obj->{shares};
        $commissionTotal += $obj->{commission};
      }
      else  # start anew.
      {
        $numSells=1;
        $masterEventRef = $obj->{masterEventRef};
        $priceTotal = $obj->{price};
        $sharesTotal = $obj->{shares};
        $commissionTotal = $obj->{commission};
        $startColor = $color;
      }
    }
    $lastColor = $color;
  }

  # we have to check for / handle the case where the last item was a sell and it was part of a run.
  if ($count > 0 && $investmentJournalViewObj->{securityEvents}[$count-1]->{eventCode} == 20 && $numSells > 1)
  {
    my $color = $startColor;
    $doc->print("  <tr bgcolor=\"$color\">\n");
    $doc->print("    <td width=\"" . ($printable eq "false" ? "95%" : "100%") . "\">\n");
    # output the totals.
    $doc->print("      <table border=\"1\" cellpadding=\"1\" cellspacing=\"0\" width=\"100%\">\n");
    $doc->print("        <tr>\n");
    $doc->print("          <td align=\"right\" width=\"44%\"><b>Totals:</b></td>\n");
    $doc->print("          <td align=\"right\" width=\"14%\"><i>" . $display->commify(value => sprintf("%.2f", $priceTotal)) . "</i></td>\n");
    $doc->print("          <td align=\"right\" width=\"14%\"><i>" . $display->commify(value => sprintf("%.2f", $sharesTotal)) . "</i></td>\n");
    $doc->print("          <td align=\"right\" width=\"15%\"><i>" . $display->commify(value => sprintf("%.2f", $commissionTotal)) . "</i></td>\n");
    $doc->print("          <td align=\"right\" width=\"13%\">&nbsp;</td>\n");
    $doc->print("        </tr>\n");
    $doc->print("      </table>\n");
    $doc->print("    </td>\n");
    if ($printable eq "false")
    {
      $doc->print("    <td width=\"5%\" align=\"center\">&nbsp;</td>\n");
    }
    $doc->print("  </tr>\n");
  }

  $doc->print(<<"END_OF_CODE");
</table>
</form>
END_OF_CODE

  # display the Legend info at the bottom of the page.
  $doc->print("<center>");
  $doc->print($self->printLegend);
  $doc->print($self->printActionCodes);
  $doc->print("</center><br><br>");

  if ($printable eq "true")
  {
    $doc->setOnload(code => "window.print()");
  }

  return $doc;
}

sub printLedgend
{
  my %legend = ( L => "Ledger Code,<br>*=Global Event",
  	         S => "Security",
  	         S2 => "Security2",
  	         A => "Account",
  	         NA => "New Account",
  	         OA => "Old Account",
  	         P => "Price",
  	         Am => "Amount",
  	         Sh => "Shares",
  	         OS => "Old Shares",
  	         NS => "New Shares",
  	         C => "Commission",
  	         Lo => "Lot",
  	         NL => "New Lot",
  	         OL => "Old Lot",
  	         "C%" => "Cost Percentage"
  	       );
  my $string;
  $string .= "<table align=\"left\" border=\"1\" cellpadding=\"1\" cellspacing=\"0\">\n";

  my $legendMax = 4;      # the max number of code to descriptions to display per row.
  my $legendCodeMax = 3;  # the max number of codes to display per row
  my $count = 0;
  foreach my $item (sort keys %legend)
  {
    if ($count == 0)
    {
      $string .= "  <tr>\n";
    }
    $string .= "    <td align=\"left\">\n";
    $string .= "      <font size=\"-1\"><b>$item</b>=$legend{$item}</font>\n";
    $string .= "    </td>\n";
    $count++;
    if ($count == $legendMax)
    {
      $string .= "  </tr>\n";
      $count = 0;
    }
  }
  if ($count != 0)
  {
    my $temp = $legendMax - $count;
    for ($count=0; $count < $temp; $count++)
    {
      $string .= "    <td>&nbsp;</td>\n";
    }
    $string .= "  </tr>\n";
  }

  $string .= "</table>\n";
  return $string;
}

sub printActionCodes
{
  my $self = shift;
  my $string;
  my $legendMax = 4;      # the max number of code to descriptions to display per row.
  my $legendCodeMax = 3;  # the max number of codes to display per row
  my $count = 0;
  $string .= "<table border=\"1\" cellpadding=\"1\" cellspacing=\"0\">\n";
  foreach my $code (sort keys %{$self->{variables}->{investmentActionHash}})
  {
    if ($count == 0)
    {
      $string .= "  <tr>\n";
    }
    $string .= "    <td align=\"left\">\n";
    $string .= "      <font size=\"-1\"><b>$code</b>=" . $self->{variables}->{investmentActionHash}->{$code} . "</font>";
    $string .= "    </td>\n";
    $count++;
    if ($count == $legendCodeMax)
    {
      $string .= "  </tr>\n";
      $count = 0;
    }
  }
  if ($count != 0)
  {
    my $temp = $legendCodeMax - $count;
    for ($count=0; $count < $temp; $count++)
    {
      $string .= "    <td>&nbsp;</td>\n";
    }
    $string .= "  </tr>\n";
  }
  $string .= "</table>\n";

}

sub inputFrame
{
  my $self = shift;
  my %args =  ( @_ );
  my $framedoc = HTMLObject::FrameSet->new();
  $framedoc->setTitle("Input Investment Transactions");

  my $inner_content;
  $inner_content = $framedoc->createNoframe("You must have a frames capable browser!");
  $inner_content .= $framedoc->createFrame(src => "$self->{getInfo}&command=inputScreen", frameborder => 'yes', scrolling => 'auto', marginwidth => '0', marginheight => '0', border => '1', name => 'inputScreen', bordercolor => 'white');
  $inner_content .= $framedoc->createFrame(src => "$self->{getInfo}&command=blank", frameborder => 'yes', scrolling => 'auto', name => 'inputdata', marginwidth => '1', bordercolor => 'white');
  $framedoc->outerFrameset(rows => '*, *', name => "inputFrame", content => $inner_content, border => '3', frameborder => 'yes');
  return $framedoc;
}

sub inputScreen
{
  my $self = shift;
  my %args =  ( inputerror => 0, @_ );
  my $phrases = $self->{langObj};

  my $account;
  my $account2;
  my $actiontype;
  my $commission;
  my $costPercentage;
  my $date;
  my $errorFields;
  my $input = $self->{input};
  my $inputerror = $args{inputerror};
  my $ledger = $self->{input}->{ledger};
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $lots;
  my $memo;
  my $newShares;
  my $oldShares;
  my $price;
  my $shares;
  my $subAccount;
  my $subAccount2;
  my $subEntity;
  my $successString;
  my $totalCost;
  my $widget = Portal::Accounting::Widgets->new(langObj => $self->{langObj});  #!!! need to consolidate widgets under portal, xiwa or something else.
  if ($widget->didErrorOccur)
  {
    $self->prefixError( $widget->errorMessage());
    return undef;
  }

  my $doc = HTMLObject::Normal->new();
  if ($doc->didErrorOccur)
  {
    $self->prefixError(errorString => $doc->errorMessage());
    return undef;
  }
  my $dbObject = Portal::Accounting::DB::Main->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
  if ($dbObject->didErrorOccur)
  {
    $self->prefixError( $dbObject->errorMessage);
    return undef;
  }
  my $portalWidget = Portal::Accounting::Widgets::DateSelect->new(langObj => $self->{langObj});
  if ($portalWidget->didErrorOccur)
  {
    $self->prefixError(errorString => $portalWidget->errorMessage());
    return undef;
  }
  my $display = Portal::Accounting::DisplayItems->new(langObj => $self->{langObj});#!!! need to consolidate widgets under portal, xiwa or something else.
  if ($display->didErrorOccur())
  {
    $self->prefixError($display->errorMessage());
    return undef;
  }
  foreach my $function ("setCommShares", "setCommission", "totalCalc")
  {
    my $string = $self->javascript(function => $function);
    if (not defined $string)
    {
      $self->prefixError("Javascript function '$function' is invalid.");
      return undef;
    }
    else
    {
      $doc->print($string);
    }
  }

  if (defined $input->{actiontype})
  {
    $actiontype = $input->{actiontype};
  }
  else
  {
    $actiontype = "buy";
  }

  if (defined $input->{date})
  {
    $date = $input->{date};
  }
  else
  {
    # get the current date and set it here
    (my $sec, my $min, my $hour, my $mday, my $mon, my $year, my $wday, my $yday, my $isdst) = localtime(time);
    $date = ((1 + $mon) < 10 ? "0" . (1 + $mon) : (1+$mon))  . "/" . ($mday < 10 ? "0" . $mday : $mday) . "/" . (1900 + $year);
  }
  $doc->setTitle($phrases->map("New Investment Transactions"));
  $doc->setFocus("body");

  $doc->print("<h2>".$phrases->map("New Investment Transaction")."</h2>\n");

  if ($inputerror == 1)
  {
    $doc->print("<p><font color=\"red\">There was an error in the input field(s): '$errorFields'<br>Please correct.</font></p>\n");
  }
  elsif ($inputerror == 2)
  {
    $doc->print("<p><font color=\"red\">Error: '$errorFields'</font></p>\n");
  }
    if (length $successString > 0)
    {
      # get the Investment Account and Sub Account numbers from the string and lookup them up.
      my ($account, $account2, $stock, $stock2);
      if ($successString =~ /ACCOUNT\(\d+\)/)
      {
        ($account = $successString) =~ s/^.*ACCOUNT\((\d+)\).*$/$1/s;
      }
      else
      {
        $account = "";
      }
      if ($successString =~ /ACCOUNT2\(\d+\)/)
      {
        ($account2 = $successString) =~ s/^.*ACCOUNT2\((\d+)\).*$/$1/s;
      }
      else
      {
        $account2 = "";
      }
      if ($successString =~ /STOCK\(\d+\)/)
      {
        ($stock = $successString) =~ s/^.*STOCK\((\d+)\).*$/$1/s;
      }
      else
      {
        $stock = "";
      }
      if ($successString =~ /STOCK2\(\d+\)/)
      {
        ($stock2 = $successString) =~ s/^.*STOCK2\((\d+)\).*$/$1/s;
      }
      else
      {
        $stock2 = "";
      }
      my $accountName = "";
      my $account2Name = "";
      my $stockName = "";
      my $stock2Name = "";

      # lookup the names to numbers.
      if (length $account > 0)
      {
        my $accountObj = $dbObject->getAccountInfo(number => $account);
        if (!defined $accountObj)
        {
          if ($dbObject->didErrorOccur)
          {
            $self->prefixError(errorString => $dbObject->errorMessage);
            return undef;
          }
          # we didn't find it
          $self->prefixError(errorString => "Account ($account) was not found!");
          return undef;
        }
        else
        {
          if ($dbObject->didErrorOccur)
          {
            $self->prefixError(errorString => $dbObject->errorMessage);
            return undef;
          }
          # we found it
          $accountName = $accountObj->name;
        }
      }
      if (length $account2 > 0)
      {
        my $accountObj = $dbObject->getAccountInfo(number => $account2);
        if (!defined $accountObj)
        {
          if ($dbObject->didErrorOccur)
          {
            $self->prefixError(errorString => $dbObject->errorMessage);
            return undef;
          }
          # we didn't find it
          $self->prefixError(errorString => "Account ($account2) was not found!");
          return undef;
        }
        else
        {
          if ($dbObject->didErrorOccur)
          {
            $self->prefixError(errorString => $dbObject->errorMessage);
            return undef;
          }
          # we found it
          $account2Name = $accountObj->name;
        }
      }
      if (length $stock > 0)
      {
        my $stockObj = $dbObject->getSubAccountInfo(number => $stock);
        if (!defined $stockObj)
        {
          if ($dbObject->didErrorOccur)
          {
            $self->prefixError(errorString => $dbObject->errorMessage);
            return undef;
          }
          # we didn't find it
          $self->prefixError(errorString => "Stock ($stock) was not found!");
          return undef;
        }
        else
        {
          if ($dbObject->didErrorOccur)
          {
            $self->prefixError(errorString => $dbObject->errorMessage);
            return undef;
          }
          # we found it
          $stockName = $stockObj->name;
        }
      }
      if (length $stock2 > 0)
      {
        my $stockObj = $dbObject->getSubAccountInfo(number => $stock2);
        if (!defined $stockObj)
        {
          if ($dbObject->didErrorOccur)
          {
            $self->prefixError(errorString => $dbObject->errorMessage);
            return undef;
          }
          # we didn't find it
          $self->prefixError(errorString => "Stock ($stock2) was not found!");
          return undef;
        }
        else
        {
          if ($dbObject->didErrorOccur)
          {
            $self->prefixError(errorString => $dbObject->errorMessage);
            return undef;
          }
          # we found it
          $stock2Name = $stockObj->name;
        }
      }

      # $doc->print("Account = '$account', Account2 = '$account2', Stock = '$stock', Stock2 = '$stock2'.<br>\n");

      # now remove the numbers from the result string and replace with their names.
      $successString =~ s/ACCOUNT\($account\)/$accountName/;
      $successString =~ s/ACCOUNT2\($account2\)/$account2Name/;
      $successString =~ s/STOCK\($stock\)/$stockName/;
      $successString =~ s/STOCK2\($stock2\)/$stock2Name/;

      # now print the successString.
      $doc->print("<font color=\"red\">Status:&nbsp;</font>&nbsp;" . $successString);
    }
    $doc->print("<p>\n");
    $doc->print("<form method=\"POST\" name=\"mainForm\">\n");
    $doc->print("<input type=\"hidden\" name=\"command\" value=\"insert\">\n");
    $doc->print($self->{postInfo});
    $doc->print("<table>\n<tr>\n");
    $doc->print("  <td>\n");
    $doc->print("  Action\n");
    $doc->print("  <select name=\"actiontype\" onChange=\"document.mainForm.target='_self'; document.mainForm.command.value='inputScreen'; document.mainForm.submit(); return true;\">\n");
    my %actionhash;
    %actionhash = %{$self->{variables}->{investmentActionHash}};
    my %numberhash;
    %numberhash = %{$self->{variables}->{investmentNumberHash}};
    my $hashIndex;
    foreach $hashIndex (sort keys %actionhash)
    {
      $doc->print("    <option");
      if ($hashIndex eq $actiontype)
      {
        $doc->print(" selected");
      }
      $doc->print(" value=\"$hashIndex\">$actionhash{$hashIndex}\n");
    }
    $doc->print("  </select>\n");
    $doc->print("  </td><td valign=\"top\">\n");
    my $output = $portalWidget->display(formName=> "mainForm", date => $date, displayStyle =>"1", dateStyle => "mdy", name => "date");
    if (! defined $output) #$portalWidget->display(doc=>$doc, formname=> "mainForm", date => $date, name => "date"))
    {
        $self->prefixError(errorString => $portalWidget->errorMessage());
        return undef;
    }
    else
    {
      $doc->print($output);
    }
    $doc->print("  </td>\n");
    $doc->print("</tr></table>\n");
    $doc->print("</p>\n");

    if (!$display->javascript_floatify(doc => $doc) )
    {
      $self->prefixError(errorString => $display->errorMessage());
      return undef;
    }
    $doc->setFocus("body");
    if ($actiontype eq "swap" || $actiontype eq "spof" || $actiontype eq "acqs")
    {
       if (defined $input->{newShares})
       {
         $newShares = $input->{newShares};
       }
       if (defined $input->{oldShares})
       {
         $oldShares = $input->{oldShares};
       }
       if (defined $input->{memo})
       {
         $memo = $input->{memo};
       }
       if (defined $input->{subAccount})
       {
         $subAccount = $input->{subAccount};
       }
       if (defined $input->{subAccount2})
       {
         $subAccount2 = $input->{subAccount2};
       }
       if (defined $input->{totalCost})
       {
         $totalCost = $input->{totalCost};
       }
       if (defined $input->{costPercentage})
       {
         $costPercentage = $input->{costPercentage};
       }
#       if (defined $input->{recordDate})
#       {
#         $recordDate = $input->{recordDate};
#       }

       $doc->print("<p>You  will receive on the date above \n");
       $doc->print("<input type=\"text\" name=\"newShares\" size=\"5\" value=\"$newShares\" onChange=\"this.value = floatify(this.value, 6); return true;\">");
       $doc->print(" new share(s) of ");
      #grab list of stocks from database
      my @list;
      @list = $dbObject->getListOfSubAccounts(type => 70, orderBy => 'sub_acc_name');
      #pass list to Widget->printSelect
      if (!$widget->printSelect(doc => $doc, name=> "subAccount2", data => \@list, selected => [$subAccount2]) )
      {
        $self->prefixError(errorString => $widget->errorMessage());
        return undef;
      }
       $doc->setFocus("body");
       $doc->print("for every ");
       $doc->print("<input type=\"text\" name=\"oldShares\" size=\"5\" value=\"$oldShares\" onChange=\"this.value = floatify(this.value, 6); return true;\">");
       $doc->print(" share(s) of ");
      #grab list of stocks from database
#      my @list;
#      @list = $dbObject->getListOfSubAccounts(type => 70);
      #pass list to Widget->displaySelect
      if (!$widget->printSelect(doc => $doc, name=> "subAccount", data => \@list, selected => [$subAccount]) )
      {
        $self->prefixError(errorString => $widget->errorMessage());
        return undef;
      }
       $doc->setFocus("body");
       $doc->print(". <br>");
       if ($actiontype eq "acqs")
       {
         $doc->print("<input type=\"hidden\" value=\"100.0\" name=\"costPercentage\">100.0");
       }
       else
       {
         $doc->print("<input type=\"text\" size=\"5\" name=\"costPercentage\" value=\"$costPercentage\" onChange=\"this.value = floatify(this.value, 6); return true;\">");
       }
       $doc->print("% of the cost of each lot will be moved from the second to a new lot of the first security");
       if ($actiontype eq "swap")
       {
         $doc->print(" and Vice Versa\n");
       }
       $doc->print(".</p>");
#       $doc->print("<p>Memo ");
#       $doc->print("<input type=\"text\" name=\"memo\" size=\"20\" value=\"$memo\">\n");
#       $doc->print("</p>\n");

    }
    elsif ($actiontype eq "splt" || $actiontype eq "sdiv")
    {
       if (defined $input->{newShares})
       {
         $newShares = $input->{newShares};
       }
       if (defined $input->{oldShares})
       {
         $oldShares = $input->{oldShares};
       }
       if (defined $input->{memo})
       {
         $memo = $input->{memo};
       }
       if (defined $input->{subAccount})
       {
         $subAccount = $input->{subAccount};
       }
       $doc->print("Security: ");
      #grab list of stocks from database
      my @list;
      @list = $dbObject->getListOfSubAccounts(type => 70, orderBy => 'sub_acc_name');
      #pass list to Widget->displaySelect
      if (!$widget->printSelect(doc => $doc, name=> "subAccount", data => \@list, selected => [$subAccount]) )
      {
        $self->prefixError(errorString => $widget->errorMessage());
        return undef;
      }
       $doc->setFocus("body");
       $doc->print("<p>You will have a total of ");
       $doc->print("<input type=\"text\" name=\"newShares\" size=\"5\" value=\"$newShares\" onChange=\"this.value = floatify(this.value, 6); return true;\">");
       $doc->print(" share(s) for every ");
       $doc->print("<input type=\"text\" name=\"oldShares\" size=\"5\" value=\"$oldShares\" onChange=\"this.value = floatify(this.value, 6); return true;\">");
       $doc->print(" original share that you own</p>");
#       $doc->print("<p>Memo ");
#       $doc->print("<input type=\"text\" name=\"memo\" size=\"20\" value=\"$memo\">\n");
#       $doc->print("</p>\n");

    }
    elsif ($actiontype eq "memo")
    {
       if (defined $input->{subAccount})
       {
         $subAccount = $input->{subAccount};
       }
       if (defined $input->{memo})
       {
         $memo = $input->{memo};
       }
       $doc->print("<p>Memo ");
       $doc->print("<input type=\"text\" name=\"memo\" value=\"$memo\"> for security \n");
       #grab list of stocks from database
       my @list = $dbObject->getListOfSubAccounts(type => 70, orderBy => 'sub_acc_name');
       #pass list to Widget->displaySelect
       if (!$widget->printSelect(doc => $doc, name=> "subAccount", data => \@list, selected => [$subAccount]) )
       {
         $self->prefixError(errorString => $widget->errorMessage());
         return undef;
       }
       $doc->print("</p>\n");

    }
    elsif ($actiontype eq "sxfr")
    {
       if (defined $input->{subAccount})
       {
         $subAccount = $input->{subAccount};
       }
       if (defined $input->{account})
       {
         $account = $input->{account};
       }
       if (defined $input->{account2})
       {
         $account2 = $input->{account2};
       }
       if (defined $input->{lots})
       {
         $lots = $input->{lots};
       }
       if (defined $input->{shares})
       {
         $shares = $input->{shares};
       }
       if (defined $input->{subEntity})
       {
         $subEntity = $input->{subEntity};
       }
       $doc->print("Transfer from account: ");
       #grab list of investment accounts from database
       my @list = $dbObject->getListOfAccounts(location => "local", type => 3, ledgerCode => $ledgerCode);
       if ($dbObject->didErrorOccur)
       {
         $self->prefixError(errorString => $dbObject->errorMessage);
         return undef;
       }
       #pass list to Widget->displaySelect
       if (!$widget->printSelect(doc => $doc, name=> "account", data => \@list, selected => [$account], onChange => "document.mainForm.lots.value = '';document.mainForm.shares.value = ''; return true;") )
       {
         $self->prefixError(errorString => $widget->errorMessage());
         return undef;
       }
       $doc->setFocus("body");
       $doc->print("; security: ");
       #select all sub accounts of type Security from DB to grab list of stocks from database
       my @list = $dbObject->getListOfSubAccounts(type => "7,70", orderBy => 'sub_acc_name');
       if ($dbObject->didErrorOccur)
       {
         $self->prefixError(errorString => $dbObject->errorMessage);
         return undef;
       }
       #pass list to Widget->displaySelect
       if (!$widget->printSelect(doc => $doc, name=> "subAccount", data => \@list, selected => [$subAccount], onChange => "document.mainForm.lots.value = ''; document.mainForm.shares.value = ''; return true;") )
       {
        $self->prefixError(errorString => $widget->errorMessage());
        return undef;
       }
       $doc->setFocus("body");
       my $button;
       $button = "<input type=\"button\" onClick=\"window.open('$self->{getInfo}&command=sharesSelect";
       $button .= "&account=' + document.mainForm.account.options[document.mainForm.account.selectedIndex].value";
       $button .= " + '&subAccount=' + document.mainForm.subAccount.options[document.mainForm.subAccount.selectedIndex].value";
       $button .= " + '&date=' + document.mainForm.date.value";
       $button .= ", 'sharesSelection', 'toolbars=no,location=no,status=no,width=400,height=400,resizable,scrollbars');\" value=\"L\" alt=\"Select Lots\">\n";
       $doc->print($button);
       $doc->print("<input type=\"hidden\" name=\"lots\" value=\"$lots\">\n");
       $doc->print("; shares: ");
       $doc->print("<input type=\"text\" name=\"shares\" size=\"8\" value=\"$shares\" onChange=\"this.value = floatify(this.value, 6); return true;\">");
       $doc->print("<br>to Sub Entity");
         #put in list of sub Entities
         my @list = $dbObject->getListOfSubEntities(ledgerCode => $ledgerCode);
         if ($dbObject->didErrorOccur)
         {
           $self->prefixError(errorString => $dbObject->errorMessage);
           return undef;
         }
         #pass list to Widget->displaySelect
         if (!$widget->printSelect(doc => $doc, name=> "subEntity", data => \@list, noneString => "--", noneValue=> "--", selected => [$subEntity] ) )
         {
           $self->prefixError(errorString => $widget->errorMessage());
           return undef;
         }
       $doc->print("in Account");
       #grab list of investment accounts from database
       my @list = $dbObject->getListOfAccounts(location => "local", type => 3, ledgerCode => $ledgerCode);
       if ($dbObject->didErrorOccur)
       {
         $self->prefixError(errorString => $dbObject->errorMessage);
         return undef;
       }
       #pass list to Widget->displaySelect
       if (!$widget->printSelect(doc => $doc, name=> "account2", data => \@list, selected => [$account2]) )
       {
         $self->prefixError(errorString => $widget->errorMessage());
         return undef;
       }
       $doc->setFocus("body");

       $doc->print(".\n<br>");
    }
    elsif ($actiontype eq "cxfr")
    {

    }
    elsif ($actiontype eq "fee")
    {
       if (defined $input->{subAccount})
       {
         $subAccount = $input->{subAccount};
       }
       if (defined $input->{account})
       {
         $account = $input->{account};
       }
       if (defined $input->{totalCost})
       {
         $totalCost = $input->{totalCost};
       }
       if (defined $input->{lots})
       {
         $lots = $input->{lots};
       }
       $doc->setFocus("body");
       $doc->print("For company ");
       #select all sub accounts of type Security from DB
       #grab list of stocks from database
       my @list = $dbObject->getListOfSubAccounts(type => "7,70", orderBy => 'sub_acc_name');
       #pass list to Widget->displaySelect
       if (!$widget->printSelect(doc => $doc, name=> "subAccount", data => \@list, selected => [$subAccount]) )
       {
         $self->prefixError(errorString => $widget->errorMessage());
         return undef;
       }
       $doc->setFocus("body");
       $doc->print("in account ");
       #grab list of investment accounts from database
       my @list = $dbObject->getListOfAccounts(location => "local", type => 3, ledgerCode => $ledgerCode);
       #pass list to Widget->displaySelect
       if (!$widget->printSelect(doc => $doc, name=> "account", data => \@list, selected => [$account]) )
       {
         $self->prefixError(errorString => $widget->errorMessage());
         return undef;
       }
       $doc->setFocus("body");
       $doc->print("for Lot number ");
       $doc->print("<input type=\"text\" name=\"lots\" value=\"$lots\" size=\"5\">\n");
#       $doc->print("Memo: <input type=\"text\" name=\"memo\" value=\"$memo\"></p>\n");
       my $button = "<input type=\"button\" onClick=\"window.open('$self->{getInfo}&command=lotSelect";
       $button .= "&account=' + document.mainForm.account.options[document.mainForm.account.selectedIndex].value";
       $button .= " + '&subAccount=' + document.mainForm.subAccount.options[document.mainForm.subAccount.selectedIndex].value";
       $button .= " + '&date=' + document.mainForm.date.value";
       $button .= ", 'lotSelection', 'toolbars=no,location=no,status=no,width=400,height=400,resizable,scrollbars');\" value=\"L\"><br>\n";
       $doc->print($button);
       $doc->setFocus("body");
       $doc->print(", I will make an adjustment to increase or decrease the cost by");
       $doc->print(" \$" . "<input type=\"text\" name=\"totalCost\" size=\"10\" value=\"$totalCost\" onChange=\"this.value = floatify(this.value, 2); return true;\">");
       $doc->print(".<br></p>\n");
    }
    elsif ($actiontype eq "rtcp")
    {
       if (defined $input->{totalCost})
       {
         $totalCost = $input->{totalCost};
       }
       if (defined $input->{subAccount})
       {
         $subAccount = $input->{subAccount};
       }
       $doc->print("The security ");
       #select all sub accounts of type Security from DB
      #grab list of stocks from database
      my @list = $dbObject->getListOfSubAccounts(type => 70, orderBy => 'sub_acc_name');
      #pass list to Widget->displaySelect
      if (!$widget->printSelect(doc => $doc, name=> "subAccount", data => \@list, selected => [$subAccount]) )
      {
        $self->prefixError(errorString => $widget->errorMessage());
        return undef;
      }
       $doc->setFocus("body");
       $doc->print(" is paying\n");
       $doc->print("\$" . "<input type=\"text\" name=\"totalCost\" size=\"10\" onChange=\"this.value = floatify(this.value, 6); return true;\"> for each share to reduce the cost in that security.<br>\n");
#       $doc->print("Memo: <input type=\"text\" name=\"memo\" value=\"$memo\">");
       $doc->print("</p>\n");
    }
    elsif ($actiontype eq "buy" || $actiontype eq "sell")
    {  #display buy/sell screen with action in form
       #see if data was sent in to display in form elements
       if (defined $input->{shares})
       {
         $shares = $input->{shares};
       }
       if (defined $input->{commission})
       {
         $commission = $input->{commission};
       }
       if (defined $input->{price})
       {
         $price = $input->{price};
       }
       if (defined $input->{memo})
       {
         $memo = $input->{memo};
       }
       if (defined $input->{totalCost})
       {
         $totalCost = $input->{totalCost};
       }
       if (defined $input->{lots})
       {
         $lots = $input->{lots};
       }
       if (defined $input->{account})
       {
         $account = $input->{account};
       }
       if (defined $input->{subAccount})
       {
         $subAccount = $input->{subAccount};
       }
       if (defined $input->{subEntity})
       {
         $subEntity = $input->{subEntity};
       }
       $doc->print("<br><table cellpadding=\"5\" cellspacing=\"5\">\n<tr>\n");
       $doc->print("  <td>\n");
       $doc->print("    Account: ");
       $doc->print("  </td>\n");
       $doc->print("  <td>\n");
       #grab list of investment accounts from database
       my @list = $dbObject->getListOfAccounts(location => "local", type => 3, ledgerCode => $ledgerCode);
       #pass list to Widget->displaySelect
       if (!$widget->printSelect(doc => $doc, name=> "account", data => \@list, selected => [$account], onChange => "document.mainForm.lots.value = ''; return true;") )
       {
         $self->prefixError(errorString => $widget->errorMessage());
         return undef;
       }
       $doc->setFocus("body");
       $doc->print("  </td>\n");
       $doc->print("  <td>\n");
       #select all sub accounts of type Security from DB
       $doc->print("    Security:");
       $doc->print("  </td>\n");
       $doc->print("  <td>\n");
       #grab list of stocks from database
       my @list = $dbObject->getListOfSubAccounts(type => "7,70", orderBy => 'sub_acc_name');
       #pass list to Widget->displaySelect
       if (!$widget->printSelect(doc => $doc, name=> "subAccount", data => \@list, selected => [$subAccount], onChange => "document.mainForm.lots.value = ''; return true;") )
       {
         $self->prefixError(errorString => $widget->errorMessage());
         return undef;
       }
       my $button = "";
       if ($actiontype eq "sell")
       {
         $button = "<input type=\"button\" onClick=\"window.open('$self->{getInfo}&command=sharesSelect";
         $button .= "&ledger=$ledger&ledgerCode=$ledgerCode&account=' + document.mainForm.account.options[document.mainForm.account.selectedIndex].value";
         $button .= " + '&subAccount=' + document.mainForm.subAccount.options[document.mainForm.subAccount.selectedIndex].value";
         $button .= " + '&date=' + document.mainForm.date.value";
         $button .= ", 'sharesSelection', 'toolbars=no,location=no,status=no,width=640,height=400,resizable,scrollbars');\" value=\"L\" alt=\"Select Lots\">";
         $doc->print($button);
         $doc->print("  </td>\n");
         $doc->print("</tr>\n");
         if (!$inputerror && $totalCost ne "")
         {
           $totalCost = $shares * $price - $commission;
         }
       }
       else
       {
         if (!$inputerror && $totalCost ne "")
         {
           $totalCost = $shares * $price + $commission;
         }
         $doc->print("  </td>\n");
         $doc->print("</tr>\n");

         $doc->print("  <td>\n");
         $doc->print("    Sub Entity: ");
         $doc->print("  </td>\n");
         $doc->print("  <td>\n");
         #put in list of sub Entities
         my @list = $dbObject->getListOfSubEntities(ledgerCode => $ledgerCode);
         if ($dbObject->didErrorOccur)
         {
           $self->prefixError(errorString => $dbObject->errorMessage);
           return undef;
         }
         #pass list to Widget->displaySelect
         if (!$widget->printSelect(doc => $doc, name=> "subEntity", data => \@list, noneString=>"--", noneValue=>"--", selected => [$subEntity]) )
         {
           $self->prefixError(errorString => $widget->errorMessage());
           return undef;
         }
         $doc->print("  </td>\n");
         $doc->print("  <td>&nbsp;\n");
         $doc->print("  </td>\n");
         $doc->print("  <td>&nbsp;\n");
         $doc->print("  </td>\n");
         $doc->print("</tr>\n");
       }
       $doc->print("<tr>\n");
       $doc->print("  <td>\n");
       $doc->print("     Shares:");
       $doc->print("  </td>\n");
       $doc->print("  <td>\n");
       $doc->print("<input type=\"text\" name=\"shares\" size=\"8\" value=\"$shares\" onChange=\"this.value = floatify(this.value, 6);totalCalc(this); return true;\">\n");
       $doc->print("  </td>\n");
       $doc->print("  <td>\n");
       $doc->print("     Price:");
       $doc->print("  </td>\n");
       $doc->print("  <td>\n");
       $doc->print(" <input type=\"text\" name=\"price\" size=\"8\" value=\"$price\" onChange=\"this.value = floatify(this.value, 6);totalCalc(this); return true;\">\n");
       $doc->print("  </td>\n");
       $doc->print("</tr>\n");
       $doc->print("<tr>\n");
       $doc->print("  <td>\n");
       $doc->print("     Comm/Share:");
       $doc->print("  </td>\n");
       $doc->print("  <td>\n");
       $doc->print("<input type=\"text\" name=\"commshares\" size=\"8\"");
       my $cs = "";
       if ($commission > 0 && $shares > 0)
       {
         $cs = sprintf("%.2f", $commission/$shares);
       }
       $doc->print(" value=\"$cs\"");
       $doc->print(" onChange=\"this.value = floatify(this.value, 6); setCommission(); totalCalc(this); return true;\">\n");
       $doc->print("  </td>\n");
       $doc->print("  <td>\n");
       $doc->print("     Commission:");
       $doc->print("  </td>\n");
       $doc->print("  <td>\n");
       $doc->print(" <input type=\"text\" name=\"commission\" size=\"8\" value=\"$commission\" onChange=\"this.value = floatify(this.value, 2); totalCalc(this); return true;\">\n");
       $doc->print("  </td>\n");
       $doc->print("</tr>\n");
       $doc->print("<tr>\n");
       $doc->print("  <td>\n");
       $doc->print("     Total:");
       $doc->print("  </td>\n");
       $doc->print("  <td>\n");
       $doc->print("<input type=\"text\" name=\"totalCost\" size=\"10\" value=\"$totalCost\" onChange=\"this.value = floatify(this.value, 2);totalCalc(this); return true;\">\n");
       $doc->print("  </td>\n");
       $doc->print("  <td>\n");
       $doc->print("     Memo:");
       $doc->print("  </td>\n");
       $doc->print("  <td>\n");
       $doc->print(" <input type=\"text\" name=\"memo\" value=\"$memo\">\n");
       $doc->print("  </td>\n");
       $doc->print("</tr>\n");
       $doc->print("</table>\n");
       $doc->print("<input type=\"hidden\" name=\"lots\" value=\"$lots\">\n");

    }

    $doc->setFocus("body");
    $doc->print("<br><input type=\"Submit\" value=\"Enter\">\n");
    if ($actiontype eq "buy" || $actiontype eq "sell")
    {
       $doc->print(" &nbsp;&nbsp;&nbsp;&nbsp;AutoChange: <input type=\"radio\" name=\"autoChange\" value=\"shares\"\>Shares");
       $doc->print(" <input type=\"radio\" name=\"autoChange\" value=\"price\">Price\n");
       $doc->print(" <input type=\"radio\" name=\"autoChange\" value=\"commission\" CHECKED>Comm\n");
       $doc->print(" <input type=\"radio\" name=\"autoChange\" value=\"total\">Total<br>\n");
    }
    $doc->print("</form>\n");

    $doc->print("<a href=\"$self->{getInfo}&command=display\">".$phrases->map("Investment Journal")."</a>\n");
    $doc->setLinkDecorations;

    return $doc;
}

sub javascript
{
  my $self = shift;
  my %args = (function => "", @_);
  my %functions;

  if ($args{function} eq "")
  {
    $self->prefixError("javascript function empty.  Please specify<br>\n");
    return undef;
  }
  $functions{setCommShares} = <<"END_OF_JAVASCRIPT";
//setCommShares: takes the current values of commission and shares an updates commshares accordingly
//pre: none
//post: none
function setCommShares()
{
  var shares = document.mainForm.shares.value;
  var commission = document.mainForm.commission.value;
  if (shares != '' && shares > 0 && commission != '' && commission > 0)
  {
    commshares = commission/shares;
    if (isNaN(commshares))
    {
      document.mainForm.commshares.value = '';
    }
    else
    {
      document.mainForm.commshares.value = floatify(commshares, 6);
      return;
    }
  }
  document.mainForm.commshares.value = '';
}
END_OF_JAVASCRIPT

 $functions{setCommission} = <<"END_OF_JAVASCRIPT";
//setCommission: takes the current values of commisshares and shares an updates commission accordingly
//pre: none
//post: none
function setCommission()
{
  var shares = document.mainForm.shares.value;
  var commshares = document.mainForm.commshares.value;
  if (shares != '' && shares > 0 && commshares != '' && commshares > 0)
  {
    commission = commshares * shares;
    if (isNaN(commission))
    {
      document.mainForm.commshares.value = '';
    }
    document.mainForm.commission.value = floatify(commission, 2);
    return;
  }
  document.mainForm.commission.value = '';
//  alert ('setting commission to nothing');
}
END_OF_JAVASCRIPT

  $functions{totalCalc} = <<"END_OF_JAVASCRIPT";
//totalCalc: based on data input so far calculate the rest of the data
//pre: textfield
//post: returns 1 for success 0 for failure
function totalCalc(textfield)
{
  var counter = 0;
  var name = textfield.name;
  var val = textfield.value;
  var shares = document.mainForm.shares.value;
  var price = document.mainForm.price.value;
  var commission = document.mainForm.commission.value;
  var commshares = document.mainForm.commshares.value;
  var totalCost = document.mainForm.totalCost.value;
  var notset = '';

  if (name == 'commshares')
  {//convert commshares to commission for calculations
    name = 'commission';
  }
  if (commission != '')
  {
    counter++;
  }
  else
  {
    notset = 'commission';
  }
  if (shares != '')
  {
    counter++;
  }
  else
  {
    notset = 'shares';
  }
  if (price != '')
  {
    counter++;
  }
  else
  {
    notset = 'price';
  }
  if (totalCost != '')
  {
    counter++;
  }
  else
  {
    notset = 'totalCost';
  }

  if (counter < 4 && name == notset && val == '')
    {
      return;//user emptied the text field
    }

//if 3 arguments are set we can calculate the fourth.  doesn''t count commshares
  if (counter == 3)
  {
    if (notset == 'commission')
    {
      if (document.mainForm.actiontype.options[document.mainForm.actiontype.selectedIndex].value == 'buy')
      {
        commission = totalCost - (shares * price);
      }
      else
      {
        commission = (shares * price) - totalCost;
      }

      if (isNaN(commission)) ///commission is not a valid number
      {
        document.mainForm.commission.value = '';
        document.mainForm.commshares.value = '';
        return 1;
      }
      else
      {
        commshares = commission / shares;
        if (isNaN(commshares))
        {
          document.mainForm.commshares.value = '';
        }
        else
        {
          document.mainForm.commshares.value = floatify(commshares, 10);
          document.mainForm.commission.value = floatify(commission, 2);
        }
      }
      return 1;
    }
    if (notset == 'shares')
    {
      if (document.mainForm.actiontype.options[document.mainForm.actiontype.selectedIndex].value == 'buy')
      {
        shares = (totalCost - commission)/ price;
      }
      else
      {
        eval("shares = (totalCost - -commission)/ price;");
      }
      if (isNaN(shares)) ///not a valid number
      {
        document.mainForm.shares.value = '';
        return 1;
      }
      else
      {
        document.mainForm.shares.value = floatify(shares, 6);
      }
      return 1;
    }
    if (notset == 'price')
    {
      if (document.mainForm.actiontype.options[document.mainForm.actiontype.selectedIndex].value == 'buy')
      {
        price = (totalCost - commission)/ shares;
      }
      else
      {
        eval("price = (totalCost - -commission)/ shares;");
      }
      if (isNaN(price)) ///not a valid number
      {
        document.mainForm.price.value = 'NaN';
        return 1;
      }
      else
      {
        document.mainForm.price.value = floatify(price, 6);
      }

      return 1;
    }
    if (notset == 'totalCost')
    {
      if (document.mainForm.actiontype.options[document.mainForm.actiontype.selectedIndex].value == 'buy')
      {
        totalCost = parseFloat(price) * parseFloat(shares) + parseFloat(commission);
      }
      else
      {
        totalCost = parseFloat(price) * parseFloat(shares) - parseFloat(commission);
        document.mainForm.memo.value = 'sell-totalCost:'+totalCost;
      }
      if (totalCost < 0)
      {
        alert( name + " calculates an invalid value! Resetting.");
        document.mainForm.totalCost.value = '';
        eval("document.mainForm." + name + ".value = '';");
        return 1;
      }
      if (isNaN(totalCost)) ///not a valid number
      {
        document.mainForm.totalCost.value = '';
        document.mainForm.memo.value = counter + ' Nan: ' + totalCost;
        return 1;
      }
      else
      {
        var temp = floatify(totalCost, 2);
        if (temp == 0)
        {
          document.mainForm.totalCost.value = '';
        }
        else
        {
          document.mainForm.totalCost.value = temp;
        }
      }
      return 1;
    }
    document.mainForm.memo.value = 'still notset:'+notset; //should never get here!
  }
  else if (counter == 4) ///four args set, ask user which one to change if total still does not add up
  { //check to see if equation holds up
    var temptotal = 0.0;
    var tempcommission = 0.0;

    if (document.mainForm.actiontype.options[document.mainForm.actiontype.selectedIndex].value == "buy")
    {
      tempcommission = commission;
    }
    else
    {
      tempcommission = 0 - commission.valueOf();
    }
    temptotal = price.valueOf() * shares.valueOf() + tempcommission;
    if (temptotal != totalCost)
    { //when total is not right use radio buttons to change the selected field.
      var autoName;
      var found = false;
      for (var i = 0; i < document.mainForm.autoChange.length && !found; i++)
      {
        if (document.mainForm.autoChange[i].checked)
        {
          found = true;
          autoName = document.mainForm.autoChange[i].value;
        }
      }
      if (autoName == "commission")
      {
        temptotal = totalCost - price*shares;
        if (temptotal < 0)
        {
          temptotal = '';//temptotal * -1;
        }
        document.mainForm.commission.value = floatify(temptotal, -1);
      }
      else if (autoName == "price")
      {
        document.mainForm.price.value = floatify( (totalCost + tempcommission)/ shares, 6 );
      }
      else if (autoName == "total")
      {
        document.mainForm.totalCost.value = floatify(temptotal, 2);
      }
      else if (autoName == "shares")
      {
        document.mainForm.shares.value = floatify( eval('('+totalCost+' + '+tempcommission+') / '+price+';'), 6 );
      }
    }
    return 1;
  }
  return 1;
}
END_OF_JAVASCRIPT

  if (not defined $functions{$args{function}})
  {
    $self->prefixError("Invalid function: '$args{function}'");
    return undef;
  }
  return $functions{$args{function}};
}

sub insert
{
  my $self = shift;
  my %args = ( @_ );
  my $account;
  my $account2;
  my $actiontype;
  my $commission;
  my $costPercentage;
  my $date;
  my $display = Portal::Accounting::DisplayItems->new(langObj => $self->{langObj});#!!! need to consolidate widgets under portal, xiwa or something else.
  if ($display->didErrorOccur())
  {
    $self->prefixError($display->errorMessage());
    return undef;
  }
  my $errorFields;
  my $inputerror;
  my $input = $self->{input};
  my $ledger = $self->{input}->{ledger};
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $lots;
  my $memo;
  my $newShares;
  my $oldShares;
  my $price;
  my $shares;
  my $subAccount;
  my $subAccount2;
  my $subEntity;
  my $successString;
  my $totalCost;
  my $dbObject = Portal::Accounting::DB::Main->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
  if ($dbObject->didErrorOccur)
  {
    $self->prefixError( $dbObject->errorMessage);
    return undef;
  }
  my $securityDBObject = $dbObject;

    $inputerror = 0;
    if (defined $input->{actiontype})
    {
      if (defined $input->{date})
      {
        $date = $input->{date};
        if ($date !~ m|(\d\d)/(\d\d)/(\d\d\d\d)|)
        {
          $inputerror = 1;
          $errorFields .= " Date";
        }
      }
      else
      {
        $inputerror = 1;
        $errorFields .= " Date";
      }

      $actiontype = $input->{actiontype};
      if ($actiontype eq "spof" || $actiontype eq "swap" || $actiontype eq "acqs")
      {
        if (defined $input->{subAccount})
        {
          $subAccount = $input->{subAccount};
          if ($subAccount eq "")
          {
            $inputerror = 1;
            $errorFields .= " Security";
          }
        }
        else
        {
          $inputerror = 1;
          $errorFields .= " Security";
        }
        if (defined $input->{subAccount2})
        {
          $subAccount2 = $input->{subAccount2};
          if ($subAccount2 eq "")
          {
            $inputerror = 1;
            $errorFields .= " Security2";
          }
        }
        else
        {
          $inputerror = 1;
          $errorFields .= " Security2";
        }
        if (defined $input->{oldShares})
        {
          $oldShares = $input->{oldShares};
          if ($oldShares !~ /^\d+\.\d+$/)
          {
            $inputerror = 1;
            $errorFields .= " Old Shares";
          }
        }
        else
        {
          $inputerror = 1;
          $errorFields .= " Old Shares";
        }
        if (defined $input->{newShares})
        {
          $newShares = $input->{newShares};
          if ($newShares !~ /^\d+\.\d+$/)
          {
            $inputerror = 1;
            $errorFields .= " New Shares";
          }
        }
        else
        {
          $inputerror = 1;
          $errorFields .= " New Shares";
        }
        if (defined $input->{costPercentage})
        {
          $costPercentage = $input->{costPercentage};
          if ($costPercentage !~ /^\d+\.\d+$/)
          {
            $inputerror = 1;
            $errorFields .= " CostPercent";
          }
        }
        else
        {
          $inputerror = 1;
          $errorFields .= " CostPercent";
        }
        if ($subAccount eq $subAccount2)
        {
          $inputerror = 1;
          $errorFields .= "<br>A Security cannot spin itself off.";
        }
        if (!$inputerror)
        {
          #change costpercentage to a decimal number
          $costPercentage /= 100.0;#/
          #take data and put in event object to give to database
          my $secObj = Portal::Accounting::Objects::SecurityEventObject->new(langObj => $self->{langObj},
              eventCode => $self->actionDeref2Number(action => $actiontype), calcDate => $date, transDate => $date,
              accountNum => $account, memoText => $memo, subAccountNum => $subAccount,
              subAccountNum2 => $subAccount2, newShares => $newShares, oldShares => $oldShares,
              costPercentage => $costPercentage);
         # counter =>   used if placing this event in between other events on same day for same subaccount
          if ($secObj->didErrorOccur())
          {
            $self->prefixError(errorString => $secObj->errorMessage());
            return undef;
          }
          #give to database
          my $eventRef;
          if ($actiontype eq "acqs")
          {
            $eventRef = $securityDBObject->insertAcquisition(secobj => $secObj);
            if (not defined $eventRef)
            {
              $self->prefixError(errorString => $securityDBObject->errorMessage());
              return undef;
            }
          }
          if ($actiontype eq "spof")
          {
            $eventRef = $securityDBObject->insertSpinOff(secobj => $secObj);
            if (not defined $eventRef)
            {
              $self->prefixError(errorString => $securityDBObject->errorMessage());
              return undef;
            }
          }
          if ($actiontype eq "swap")
          {
            $eventRef = $securityDBObject->insertStockSwap(secobj => $secObj);
            if (not defined $eventRef)
            {
              $self->prefixError(errorString => $securityDBObject->errorMessage());
              return undef;
            }
          }

          my $actionName = $self->actionDeref(action => $actiontype);
          $successString = "Entered <b>$actionName</b><br>\n";
          $successString .= "You received\n";
          $successString .= " $newShares ";
          $successString .= " new share(s) of STOCK2($subAccount2)";
          $successString .= " for every $oldShares";
          $successString .= " share(s) of STOCK($subAccount)";
          $successString .= " held on $date. <br>";
          if ($actiontype eq "acqs")
          {
            $successString .= "100";
          }
          else
          {
            $successString .= sprintf("%.2f", $costPercentage * 100);
          }
          $successString .= "% of the cost of each lot was moved from the second to a new lot of the first security";
          if ($actiontype eq "swap")
          {
            $successString .= " and Vice Versa\n";
          }
          $successString .= ".</p>";
          $input->{oldShares} = "";
          $input->{newShares} = "";
          $input->{costPercentage} = "";
          $input->{lots} = "";
#          $doc->print("<p>Memo: ");
#          $doc->print("$memo\n");
#          $doc->print("</p>\n");
        }
      }
      elsif ($actiontype eq "sdiv" || $actiontype eq "splt")
      {
        if (defined $input->{subAccount})
        {
          $subAccount = $input->{subAccount};
          if ($subAccount eq "")
          {
            $inputerror = 1;
            $errorFields .= " Security";
          }
        }
        else
        {
          $inputerror = 1;
        }
        if (defined $input->{oldShares})
        {
          $oldShares = $input->{oldShares};
          if ($oldShares !~ /^\d+\.\d+$/)
          {
            $inputerror = 1;
            $errorFields .= " Old shares";
          }
        }
        else
        {
          $inputerror = 1;
        }
        if (defined $input->{newShares})
        {
          $newShares = $input->{newShares};
          if ($newShares !~ /^\d+\.\d+$/)
          {
            $inputerror = 1;
            $errorFields .= " New shares";
          }
        }
        else
        {
          $inputerror = 1;
        }
        if (defined $input->{memo})
        {
          $memo = $input->{memo};
        }
        if (!$inputerror)
        {
          # $costPercentage /= 100.0;#/
          #take data and put in event object to give to database
          my $secObj = Portal::Accounting::Objects::SecurityEventObject->new(langObj => $self->{langObj},
              eventCode => $self->actionDeref2Number(action => $actiontype), calcDate => $date, transDate => $date,
              subAccountNum => $subAccount, newShares => $newShares, oldShares => $oldShares);
         # counter =>   used if placing this event in between other events on same day for same subaccount
          if ($secObj->didErrorOccur())
          {
            $self->prefixError(errorString => $secObj->errorMessage());
            return undef;
          }
          #give to database
          my $eventRef;
          if ($actiontype eq "sdiv")
          {
            $eventRef = $securityDBObject->insertStockDividend(secobj => $secObj);
            if (not defined $eventRef)
            {
              $self->prefixError(errorString => $securityDBObject->errorMessage());
              return undef;
            }
          }
          if ($actiontype eq "splt")
          {
            $eventRef = $securityDBObject->insertStockSplit(secobj => $secObj);
            if (not defined $eventRef)
            {
              $self->prefixError(errorString => $securityDBObject->errorMessage());
              return undef;
            }
          }

          my $actionName = $self->actionDeref(action => $actiontype);
          $successString = "Entered <b>$actionName</b><br>\n";
          $successString .= "For Security: STOCK($subAccount)<br>\n";
          $successString .= "on $date<br>\n";
          $successString .= "Each $oldShares share(s)<br>\n";
          $successString .= "yeilded a total of $newShares share(s)<br>\n";
#          $successString .= "Memo: $memo\n";
          $input->{oldShares} = "";
          $input->{newShares} = "";
          $input->{memo} = "";
          $input->{lots} = "";
        }
      }
      elsif ($actiontype eq "memo")
      {
        if (defined $input->{subAccount})
        {
          $subAccount = $input->{subAccount};
        }
        else
        {
          $inputerror = 1;
          $errorFields .= " Security";
        }
        if (defined $input->{memo} && $input->{memo} ne "")
        {
          $memo = $input->{memo};
        }
        else
        {
          $inputerror = 1;
          $errorFields .= " Memo";
        }
        if (!$inputerror)
        {
          #take data and put in event object to give to database
          my $secObj = Portal::Accounting::Objects::SecurityEventObject->new(langObj => $self->{langObj},
              eventCode => $self->actionDeref2Number(action => $actiontype), calcDate => $date, transDate => $date,
              subAccountNum => $subAccount, memoText => $memo, ledgerCode => $ledgerCode);
         # counter =>   used if placing this event in between other events on same day for same subaccount
          if ($secObj->didErrorOccur())
          {
            $self->prefixError(errorString => $secObj->errorMessage());
            return undef;
          }
          #give to database
          my $eventRef;
          if ($actiontype eq "memo")
          {
            $eventRef = $securityDBObject->insertMemo(secobj => $secObj);
            if (not defined $eventRef)
            {
              $self->prefixError(errorString => $securityDBObject->errorMessage());
              return undef;
            }
          }

          my $actionName = $self->actionDeref(action => $actiontype);
          $successString = "<p>Entered <b>$actionName</b><br>\n";
          $successString .= "on $date for STOCK($subAccount)<br>\n";
          $successString .= "$memo</p>\n";
          $input->{memo} = "";
        }
      }
      elsif ($actiontype eq "sxfr")
      {
        if (defined $input->{subAccount})
        {
          $subAccount = $input->{subAccount};
        }
        else
        {
          $inputerror = 1;
          $errorFields .= " Security";
        }
        if (defined $input->{shares})
        {
          $shares = $input->{shares};
          if ($shares eq "")
          {
            $inputerror = 1;
            $errorFields .= " Shares";
          }
        }
        else
        {
          $inputerror = 1;
          $errorFields .= " Shares";
        }
        if (defined $input->{account})
        {
          $account = $input->{account};
          if ($account eq "")
          {
            $inputerror = 1;
            $errorFields .= " Account";
          }
        }
        else
        {
          $inputerror = 1;
          $errorFields .= " Account";
        }
        if (defined $input->{account2})
        {
          $account2 = $input->{account2};
          if ($account2 eq "")
          {
            $inputerror = 1;
            $errorFields .= " Account2";
          }
        }
        else
        {
          $inputerror = 1;
          $errorFields .= " Account2";
        }
        if (defined $input->{subEntity})
        {
          $subEntity = $input->{subEntity};
          if ($subEntity eq "--")
          {
            $subEntity = "";
          }
        }
        else
        {
          $inputerror = 1;
          $errorFields .= " subEntity";
        }
        if ($account eq $account2)
        {
          $inputerror = 1;
          $errorFields .= "<br>Cannot transfer stock into the same account it is already in!";
        }
        my %lotHash;
        if (defined $input->{lots})
        {
          # run through each of the lots to be transferred and insert a transfer event into the database with the appropriate data
          # check to make sure the number of shares being transferred does not exceed the number of shares for the lot
          # in the future
            $lots = $input->{lots};
            if ($lots eq "")
            {
              $inputerror = 1;
              $errorFields .= " Lots";
            }
            else
            {
              #hashifycheck splits lot string into a hash and checks to make sure there are enough shares for each lot
              my $hashRef;
              ($inputerror, $hashRef) = $self->lotHashifyCheck(lots => $lots, eventDate=>$date, securityDBOject => $securityDBObject);
              %lotHash = %{ $hashRef };
              if ($inputerror >0 || (scalar keys %lotHash < 1))
              {
                $errorFields .= " Lots";
                $inputerror = 1;
              }
            }
        }
        else
        {
            $inputerror = 1;
            $errorFields .= " Lots";
        }
        if (!$inputerror)
        {
            # actually insert the data into the database
          foreach my $lotIndex (sort keys %lotHash)
          {
              my $secObj = undef;
              #take data and put in event object to give to database
              $secObj = Portal::Accounting::Objects::SecurityEventObject->new(langObj => $self->{langObj},
                eventCode => $self->actionDeref2Number(action => $actiontype), calcDate => $date, transDate => $date,
                subEntity => $subEntity, oldAccountNum => $account, newAccountNum => $account2, subAccountNum => $subAccount,
                oldLot => $lotIndex, shares => $lotHash{$lotIndex}, ledgerCode => $ledgerCode, subEntity => $subEntity);
             # counter =>   used if placing this event in between other events on same day for same subaccount
              if ($secObj->didErrorOccur())
              {
                $self->prefixError(errorString => $secObj->errorMessage());
                return undef;
              }
              #give to database
              my $eventRef = $securityDBObject->insertStockTransfer(secobj => $secObj);
              if ($securityDBObject->didErrorOccur())
              {
                $self->prefixError(errorString => $securityDBObject->errorMessage());
                return undef;
              }
              if (not defined $eventRef)
              {
                $self->prefixError(errorString => $securityDBObject->errorMessage());
                return undef;
              }
          }

            my $actionName = $self->actionDeref(action => $actiontype);
            $successString = "Entered <b>$actionName</b><br>\n";
            $successString .= "<table>\n";
            $successString .= "<tr>\n";
            $successString .= "<td>\n";
            $successString .= "Date: $date\n";
            $successString .= "</td>\n";
            $successString .= "</tr>\n";
            $successString .= "<tr>\n";
            $successString .= "<td>\n";
            $successString .= "From Account: ACCOUNT($account)\n";
            $successString .= "</td>\n";
            $successString .= "<td>\n";
            $successString .= "To Account: ACCOUNT2($account2)\n";
            $successString .= "</td>\n";
            $successString .= "</tr>\n";
            $successString .= "<tr>\n";
            $successString .= "<td>\n";
            $successString .= "Lots: $lots\n";
            $successString .= "</td>\n";
            $successString .= "<td>\n";
            $successString .= "Shares: $shares\n";
            $successString .= "</td>\n";
            $successString .= "</tr>\n";
#            $successString .= "<tr>\n";
#            $successString .= "<td colspan=2>\n";
#            $successString .= "Memo: $memo\n";
#            $successString .= "</td>\n";
#            $successString .= "</tr>\n";
            $successString .= "</table>\n";
            $input->{shares} = "";
            $input->{lots} = "";
        }
      }
      elsif ($actiontype eq "fee")
      {
        $inputerror = 0;
        if (defined $input->{account})
        {
          $account = $input->{account};
          if ($account eq "")
          {
            $inputerror = 1;
            $errorFields .= " Account";
          }
        }
        else
        {
          $inputerror = 1;
            $errorFields .= " Account";
        }
        if (defined $input->{subAccount})
        {
          $subAccount = $input->{subAccount};
          if ($subAccount eq "")
          {
            $inputerror = 1;
            $errorFields .= " Security";
          }
        }
        else
        {
          $inputerror = 1;
            $errorFields .= " Security";
        }
        if (defined $input->{totalCost})
        {
          $totalCost = $input->{totalCost};
          if ($totalCost !~ /^(-)?\d+\.\d+$/)
          {
            $inputerror = 1;
            $errorFields .= " Amount";
          }
        }
        else
        {
          $inputerror = 1;
            $errorFields .= " Amount";
        }
        if (defined $input->{lots})
        {
          $lots = $input->{lots};
          if ($lots !~ /^[0-9]+$/)
          {
            $inputerror = 1;
            $errorFields .= " Lot";
          }
        }
        else
        {
          $inputerror = 1;
          $errorFields .= " Lot";
        }
        if (defined $input->{memo})
        {
          $memo = $input->{memo};
        }
        if (!$inputerror)
        {
          #take data and put in event object to give to database
          my $secObj = Portal::Accounting::Objects::SecurityEventObject->new(langObj => $self->{langObj},
              eventCode => $self->actionDeref2Number(action => $actiontype), calcDate => $date, transDate => $date,
              accountNum => $account, subAccountNum => $subAccount, amount => $totalCost, lot => $lots,
              ledgerCode => $ledgerCode);
         # counter =>   used if placing this event in between other events on same day for same subaccount
          if ($secObj->didErrorOccur())
          {
            $self->prefixError(errorString => $secObj->errorMessage());
            return undef;
          }
          #give to database
          my $eventRef;
          if ($actiontype eq "fee")
          {
            $eventRef = $securityDBObject->insertFee(secobj => $secObj);
            if (not defined $eventRef)
            {
              $self->prefixError(errorString => $securityDBObject->errorMessage());
              return undef;
            }
          }

          my $actionName = $self->actionDeref(action => $actiontype);
          $successString = "Entered <b>$actionName</b><br>\n";
          $successString .= "<table>\n";
          $successString .= "<tr>\n";
          $successString .= "  <td>\n";
          $successString .= "Date: $date\n";
          $successString .= "  </td>\n";
          $successString .= "  <td>\n";
          $successString .= "Cost: $totalCost\n";
          $successString .= "  </td>\n";
          $successString .= "</tr>\n";
          $successString .= "<tr>\n";
          $successString .= "  <td>\n";
          $successString .= "Security: STOCK($subAccount)\n";
          $successString .= "  </td>\n";
          $successString .= "  <td>\n";
          $successString .= "Account: ACCOUNT($account)\n";
          $successString .= "  </td>\n";
          $successString .= "</tr>\n";
#          $successString .= "<tr>\n";
#          $successString .= "<td colspan=3>\n";
#          $successString .= "Memo: $memo\n";
#          $successString .= "</td>\n";
#          $successString .= "</tr>\n";
          $successString .= "</table>\n";
          $input->{totalCost} = "";
          $input->{lots} = "";
          $input->{memo} = "";
        }
      }
      elsif($actiontype eq "rtcp")
      {
        $inputerror = 0;
        if (defined $input->{subAccount})
        {
          $subAccount = $input->{subAccount};
          if ($subAccount eq "")
          {
            $inputerror = 1;
            $errorFields .= " Security";
          }
        }
        else
        {
          $inputerror = 1;
            $errorFields .= " Security";
        }
        if (defined $input->{totalCost})
        {
          $totalCost = $input->{totalCost};
          if ($totalCost !~ /^\d+\.\d+$/)
          {
            $inputerror = 1;
            $errorFields .= " Amount";
          }
        }
        else
        {
          $inputerror = 1;
            $errorFields .= " Amount";
        }
        if (defined $input->{memo})
        {
          $memo = $input->{memo};
        }
        if (!$inputerror)
        {
          #take data and put in event object to give to database
          my $secObj = Portal::Accounting::Objects::SecurityEventObject->new(langObj => $self->{langObj},
              eventCode => $self->actionDeref2Number(action => $actiontype), calcDate => $date, transDate => $date,
              subAccountNum => $subAccount, amount => $totalCost, ledgerCode => "*");
         # counter =>   used if placing this event in between other events on same day for same subaccount
          if ($secObj->didErrorOccur())
          {
            $self->prefixError(errorString => $secObj->errorMessage());
            return undef;
          }
          #give to database
          my $eventRef;
          if ($actiontype eq "rtcp")
          {
            $eventRef = $securityDBObject->insertReturnOfCapital(secobj => $secObj);
            if ($securityDBObject->didErrorOccur())
            {
              $self->prefixError(errorString => $securityDBObject->errorMessage());
              return undef;
            }
            if (not defined $eventRef)
            {
              $self->prefixError(errorString => $securityDBObject->errorMessage());
              return undef;
            }
          }

          my $actionName = $self->actionDeref(action => $actiontype);
          $successString = "Entered <b>$actionName</b><br>\n";
          $successString .= "<table>\n";
          $successString .= "<tr>\n";
          $successString .= "  <td colspan=\"2\">\n";
          $successString .= "Date: $date\n";
          $successString .= "  </td>\n";
          $successString .= "</tr>\n";
          $successString .= "<tr>\n";
          $successString .= "  <td>\n";
          $successString .= "Security: STOCK($subAccount)\n";
          $successString .= "  </td>\n";
          $successString .= "  <td>\n";
          $successString .= "Cost: $totalCost\n";
          $successString .= "  </td>\n";
          $successString .= "</tr>\n";
#          $successString .= "<tr>\n";
#          $successString .= "<td colspan=2>\n";
#          $successString .= "Memo: $memo\n";
#          $successString .= "</td>\n";
#          $successString .= "</tr>\n";
          $successString .= "</table>\n";
          $input->{totalCost} = "";
          $input->{memo} = "";
          $input->{lots} = "";
        }
      }
      elsif ($actiontype eq "buy" || $actiontype eq "sell")
      {
        if (defined $input->{account})
        {
          $account = $input->{account};
          if ($account eq "")
          {
            $inputerror = 1;
            $errorFields .= " Account";
          }
        }
        else
        {
          $inputerror = 1;
            $errorFields .= " Account";
        }
        if (defined $input->{subAccount})
        {
          $subAccount = $input->{subAccount};
          if ($subAccount eq "")
          {
            $inputerror = 1;
            $errorFields .= " Security";
          }
        }
        else
        {
          $inputerror = 1;
            $errorFields .= " Security";
        }
        if (defined $input->{shares})
        {
          $shares = $input->{shares};
          if ($shares !~ /^\d+\.\d+$/)
          {
            $inputerror = 1;
            $errorFields .= " Shares";
          }
        }
        else
        {
          $inputerror = 1;
            $errorFields .= " Shares";
        }
        if (defined $input->{price})
        {
          $price = $input->{price};
          if ($price !~ /^\d+\.\d+$/)
          {
            $inputerror = 1;
            $errorFields .= " Price";
          }
        }
        else
        {
          $inputerror = 1;
            $errorFields .= " Price";
        }
        if (defined $input->{commission})
        {
          $commission = $input->{commission};
          if ($commission !~ /^\d+\.\d+$/)
          {
            $inputerror = 1;
            $errorFields .= " Commission";
          }
        }
        else
        {
          $inputerror = 1;
            $errorFields .= " Commission";
        }
        if (defined $input->{totalCost})
        {
          $totalCost = $input->{totalCost};
          if ($totalCost !~ /^\d+\.\d+$/)
          {
            $inputerror = 1;
            $errorFields .= " Total";
          }
        }
        else
        {
          $inputerror = 1;
            $errorFields .= " Total";
        }
        if (defined $input->{memo})
        {
          $memo = $input->{memo};
        }
        my %lotHash;
        if ($actiontype eq "sell")
        {
          if ($totalCost != ($shares * $price) - $commission)
          {
            $inputerror = 1;
            $errorFields .= " Total Calculation";
          }
          if (defined $input->{lots})
          {
            $lots = $input->{lots};
            my $hashRef;
            ($inputerror, $hashRef) = $self->lotHashifyCheck(eventDate=>$date, lots =>$lots, securityDBObject => $securityDBObject);
            %lotHash = %{ $hashRef };
            if ($lots eq "" || $inputerror)
            {
              $inputerror = 1;
              $errorFields .= " Lots";
            }
          }
          else
          {
            $inputerror = 1;
            $errorFields .= " Lots";
          }
        }
        else
        {
          if ($totalCost != ($shares * $price) + $commission)
          {
            $inputerror = 1;
            $errorFields .= " Total Calculation";
          }
        }
        if (!$inputerror)
        {
          if ($actiontype eq "buy")
          {
            #take data and put in event object to give to database
            my $secObj = Portal::Accounting::Objects::SecurityEventObject->new(langObj => $self->{langObj},
                eventCode => $self->actionDeref2Number(action => $actiontype), calcDate => $date, transDate => $date,
                subEntity => $subEntity, accountNum => $account, subAccountNum => $subAccount,
                price => $price, shares => $shares, commission => $commission,
                ledgerCode => $ledgerCode);
           # counter =>   used if placing this event in between other events on same day for same subaccount
            if ($secObj->didErrorOccur())
            {
              $self->prefixError(errorString => $secObj->errorMessage());
              $inputerror = 1;#return undef;
              $errorFields .= "Inserting into Database<br>";
            }
            #give to database
            my $eventRef;
            $eventRef = $securityDBObject->insertBuy(secobj => $secObj);
            if ($securityDBObject->didErrorOccur())
            {
              $self->prefixError(errorString => $securityDBObject->errorMessage());
              return undef;
            }
            if (not defined $eventRef)
            {
              $self->prefixError(errorString => $securityDBObject->errorMessage());
              return undef;
            }
          }#endbuy
          elsif ($actiontype eq "sell")
          {
            my $commissionPerShare = sprintf("%.2f", ($commission / $shares)); #/
            my $leftoverCommission = sprintf("%.2f", ($commission - ($commissionPerShare * $shares)));
            # run through each of the lots to be sold and insert a sell event into the database with the appropriate data
            # check to make sure the number of shares being sold does not exceed the number of shares for the lot in the
            # future
            my $masterRef = -1;
            foreach my $lotIndex (sort keys %lotHash)
            {
                my $secObj = undef;
                #take data and put in event object to give to database
                if ($masterRef == -1)
                {
                  $secObj = Portal::Accounting::Objects::SecurityEventObject->new(langObj => $self->{langObj}, masterEventRef => $masterRef,
                    eventCode => $self->actionDeref2Number(action => $actiontype),
                    calcDate => $date, transDate => $date,
                    accountNum => $account, subAccountNum => $subAccount, lot => $lotIndex,
                    price => $price, shares => $lotHash{$lotIndex},
                    commission => sprintf("%.2f", ($commissionPerShare * $lotHash{$lotIndex} + $leftoverCommission) ),
                    ledgerCode => $ledgerCode);
                    #subEntity => $SubEntity,
                    # counter =>   used if placing this event in between other events on same day for same subaccount
                }
                else
                {
                  $secObj = Portal::Accounting::Objects::SecurityEventObject->new(langObj => $self->{langObj}, masterEventRef => $masterRef,
                    eventCode => $self->actionDeref2Number(action => $actiontype), calcDate => $date, transDate => $date,
                    subEntity => "", accountNum => $account, subAccountNum => $subAccount, lot => $lotIndex,
                    price => $price, shares => $lotHash{$lotIndex},
                    commission => sprintf("%.2f", ($commissionPerShare * $lotHash{$lotIndex}) ),
                    ledgerCode => $ledgerCode);
               # counter =>   used if placing this event in between other events on same day for same subaccount
                }
                if ($secObj->didErrorOccur())
                {
                  $self->prefixError(errorString => $secObj->errorMessage());
                  $inputerror =1;
                  $errorFields .= "Error Creating Object Sell: $lotHash{$lotIndex} shares of Lot: $lotIndex<br>";
                  #return undef;
                }
                #give to database
                my $eventRef;
                $eventRef = $securityDBObject->insertSell(secobj => $secObj);
                if ($securityDBObject->didErrorOccur())
                {
                  $self->prefixError(errorString => $securityDBObject->errorMessage());
                  $inputerror =1;
                  $errorFields .= "Error Inserting Sell: $lotHash{$lotIndex} shares of Lot: $lotIndex<br>";
                  #return undef;
                }
                if (not defined $eventRef)
                {
                  $self->prefixError(errorString => $securityDBObject->errorMessage());
                  $inputerror =1;
                  $errorFields .= "Error getting event reference for Sell: $lotHash{$lotIndex} shares of Lot: $lotIndex<br>";
#                  return undef;
                }
                elsif ($masterRef == -1)
                {
                  $masterRef = $eventRef;
                }

            }#endfor
          }#endsell

          if (!$inputerror)
          {
            my $actionName = $self->actionDeref(action => $actiontype);
            $successString .= "Entered <b>$actionName</b><br>\n";
            $successString .= "<table>\n";
            $successString .= "<tr>\n";
            $successString .= "  <td>\n";
            $successString .= "Security: STOCK($subAccount)\n";
            $successString .= "  </td>\n";
            $successString .= "  <td>\n";
            $successString .= "Account: ACCOUNT($account)\n";
            $successString .= "  </td>\n";
            $successString .= "</tr>\n";
            $successString .= "<tr>\n";
            $successString .= "  <td>\n";
            $successString .= "Shares: $shares\n";
            $successString .= "  </td>\n";
            $successString .= "  <td>\n";
            $successString .= "Price: $price\n";
            $successString .= "  </td>\n";
            $successString .= "</tr>\n";
            $successString .= "<tr>\n";
            $successString .= "  <td>\n";
            $successString .= "Comm: " . sprintf("%.2f", $commission) . "\n";
            $successString .= "  </td>\n";
            $successString .= "  <td>\n";
            $successString .= "Cost: " . sprintf("%.2f", $totalCost) . "\n";
            $successString .= "  </td>\n";
            $successString .= "</tr>\n";
            $successString .= "<tr>\n";
            $successString .= "  <td>\n";
            $successString .= "Date: $date\n";
            $successString .= "  </td>\n";
            $successString .= "  <td>\n";
            $successString .= "Memo: $memo\n";
            $successString .= "  </td>\n";
            $successString .= "</tr>\n";
            $successString .= "<tr>\n";
            if ($actiontype eq "sell")
            {
              $successString .= "<td colspan=2>\n";
              $successString .= "Lots: $lots\n";
              $successString .= "</td>\n";
              $successString .= "</tr>\n";
            }
            $successString .= "</table>\n";
            $input->{shares} = "";
            $input->{price} = "";
            $input->{totalCost} = "";
            $input->{memo} = "";
            $input->{commission} = "";
            $input->{lots} = "";
          }
        }#inputerror
      }#end buy sell
      else
      {
        $self->prefixError(errorString => "Invalid Actiontype: $actiontype\n");
        return undef;
      }
    }
    else #actiontype not defined
    {
      $self->prefixError(errorString => "Actiontype not defined\n");
      return undef;
    }
#    $command = "status"; #data not valid rerun the input screen with the default values
    return $self->status;
}

sub status
{
  my $self = shift;
  my %args =  ( @_ );
  my $doc = HTMLObject->new;
  $doc->setTitle("Status");
  $doc->setFocus("body");
  $doc->print("Status: $self->{input}->{message}");
  return $doc;
}

sub lotSelect
{
  my $self = shift;
  my %args =  ( @_ );

  my $account;
  my $date;
  my $errorFields;
  my $input = $self->{input};
  my $inputerror;
  my $ledger = $self->{input}->{ledger};
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $subAccount;

  my $dbObject = Portal::Accounting::DB::Main->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
  if ($dbObject->didErrorOccur)
  {
    $self->prefixError( $dbObject->errorMessage);
    return undef;
  }
  my $securityDBObject = $dbObject;
    #check for account and security pair that should have been sent it.
    if (defined $input->{date})
    {
      $date = $input->{date};
      if ($date eq "")
      {
        $inputerror = 1;
        $errorFields .= " 'Date'";
      }
    }
    else
    {
      $inputerror = 1;
      $errorFields .= " 'Date'";
    }
    if (defined $input->{account})
    {
      $account = $input->{account};
      if ($account eq "")
      {
        $inputerror = 1;
        $errorFields .= " 'Account'";
      }
    }
    else
    {
      $inputerror = 1;
      $errorFields .= " 'Account'";
    }
    if (defined $input->{subAccount})
    {
      $subAccount = $input->{subAccount};
      if ($subAccount eq "")
      {
        $inputerror = 1;
        $errorFields .= " 'Security'";
      }
    }
    else
    {
      $inputerror = 1;
      $errorFields .= " 'Security'";
    }
    my $doc = HTMLObject::Normal->new();
    $doc->setTitle("Lot Selection");
    $doc->setFocus("body");
    if ($inputerror)
    {
      $doc->print("<h1><font color=\"red\">Error.  Fields missing data: $errorFields</font></h1>\n");
    }
    else
    {
      #get all lots for sub account sent in
      my @lotNodes = $securityDBObject->getLotBalanceEntriesFor(accountNum => $account, subAccountNum => $subAccount, ledgerCode => $ledgerCode, order => "lot", eventDate=>$date);
      if ($securityDBObject->didErrorOccur())
      {
        $self->{errorString} .= "<font color=\"red\">SecurityDBObject had error</font><br>\n";
        $self->prefixError(errorString => $securityDBObject->errorMessage());
        return undef;
      }
      if (not defined @lotNodes)
      {#there were no lots or there was a problem
        # lookup the account and sub account names.
        my $accountObj = $dbObject->getAccountInfo(number => $account);
        my $accountName = "";
        if (!defined $accountObj)
        {
          if ($dbObject->didErrorOccur)
          {
            $self->prefixError(errorString => $dbObject->errorMessage);
            return undef;
          }
          # we didn't find it
          $self->prefixError(errorString => "Account ($account) was not found!");
          return undef;
        }
        else
        {
          if ($dbObject->didErrorOccur)
          {
            $self->prefixError(errorString => $dbObject->errorMessage);
            return undef;
          }
          # we found it
          $accountName = $accountObj->name;
        }
        my $stockObj = $dbObject->getSubAccountInfo(number => $subAccount);
        my $subAccountName = "";
        if (!defined $stockObj)
        {
          if ($dbObject->didErrorOccur)
          {
            $self->prefixError(errorString => $dbObject->errorMessage);
            return undef;
          }
          # we didn't find it
          $self->prefixError(errorString => "Stock ($subAccount) was not found!");
          return undef;
        }
        else
        {
          if ($dbObject->didErrorOccur)
          {
            $self->prefixError(errorString => $dbObject->errorMessage);
            return undef;
          }
          # we found it
          $subAccountName = $stockObj->name;
        }
        $doc->print("<p><font color=\"red\">There were no lots found for this Account ($accountName) and Security ($subAccountName) and Ledger ($ledgerCode) and Date ($date).</font></p>");
        $doc->print("<form>\n");
        $doc->print("<input type=\"button\" value=\"Close\" onClick=\"self.close();\">\n");
        $doc->print("</form>\n");
      }
      else
      {
        $doc->print("<p>Select the lot to which to apply the adjustment.</p>");
        $doc->print("<form name=\"lot_form\">\n");
        $doc->print("<select name=\"lot_select\">\n");
        for (my $lotCount=0; $lotCount < scalar @lotNodes; $lotCount++)
        {
          $doc->print("  <option>" . $lotNodes[$lotCount]->lot . "\n");
        }
        $doc->print("</select>\n");
        $doc->print("<input type=\"button\" value=\"Done\" onClick=\"writeLots();self.close();\">\n");
        $doc->print("</rform>");
      }
      $doc->setFocus("javascript");
      $doc->print("function writeLots()\n");
      $doc->print("{//write data back into parent window into mainForm.lots\n");
      $doc->print("  var lotstring = '';\n");
      $doc->print("  lotstring = document.lot_form.lot_select.options[document.lot_form.lot_select.selectedIndex].text;\n");
      $doc->print("  var windowref = window.opener;\nwindowref.document.mainForm.lots.value = lotstring;\n");
      $doc->print("}\n");
    }
    return $doc;
}

sub sharesSelect
{
  my $self = shift;
  my %args =  ( @_ );
  my $account;
  my $actiontype;
  my $costPercentage;
  my $date;
  my $display = Portal::Accounting::DisplayItems->new(langObj => $self->{langObj});#!!! need to consolidate widgets under portal, xiwa or something else.
  if ($display->didErrorOccur())
  {
    $self->prefixError($display->errorMessage());
    return undef;
  }
  my $errorFields;
  my $inputerror;
  my $input = $self->{input};
  my $ledger = $self->{input}->{ledger};
  my $ledgerCode = $self->{input}->{ledgerCode};
  my $memo;
  my $newShares;
  my $oldShares;
  my $subAccount;
  my $subAccount2;
  my $dbObject = Portal::Accounting::DB::Main->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
  if ($dbObject->didErrorOccur)
  {
    $self->prefixError( $dbObject->errorMessage);
    return undef;
  }
  my $securityDBObject = $dbObject;
    #check for account and security pair that should have been sent it.
    if (defined $input->{date})
    {
      $date = $input->{date};
      if ($date eq "")
      {
        $inputerror = 1;
        $errorFields .= " 'Date'";
      }
    }
    else
    {
      $inputerror = 1;
      $errorFields .= " 'Date'";
    }
    if (defined $input->{account})
    {
      $account = $input->{account};
      if ($account eq "")
      {
        $inputerror = 1;
        $errorFields .= " 'Account'";
      }
    }
    else
    {
      $inputerror = 1;
      $errorFields .= " 'Account'";
    }
    if (defined $input->{subAccount})
    {
      $subAccount = $input->{subAccount};
      if ($subAccount eq "")
      {
        $inputerror = 1;
        $errorFields .= " 'Security'";
      }
    }
    else
    {
      $inputerror = 1;
      $errorFields .= " 'Security'";
    }
    my $doc = HTMLObject::Normal->new();
    $doc->setTitle("Shares Selection");
    $doc->setFocus("body");
    if ($inputerror)
    {
      $doc->print("<h1><font color=\"red\">Error.  Fields missing data: $errorFields</font></h1>\n");
    }
    else
    {
      #get all lots for account, sub account pair sent in
      my @lotNodes = $securityDBObject->getLotBalanceEntriesFor(accountNum => $account, subAccountNum => $subAccount, ledgerCode => $ledgerCode, order => "lot", eventDate=>$date);
      if ($securityDBObject->didErrorOccur())
      {
        $self->{errorString} .= "<font color=\"red\">SecurityDBObject had error</font><br>\n";
        $self->prefixError(errorString => $securityDBObject->errorMessage());
        return undef;
      }
      if (not defined @lotNodes)
      {#there were no lots or there was a problem
        # lookup the account and sub account names.
        my $accountObj = $dbObject->getAccountInfo(number => $account);
        my $accountName = "";
        if (!defined $accountObj)
        {
          if ($dbObject->didErrorOccur)
          {
            $self->prefixError(errorString => $dbObject->errorMessage);
            return undef;
          }
          # we didn't find it
          $self->prefixError(errorString => "Account ($account) was not found!");
          return undef;
        }
        else
        {
          if ($dbObject->didErrorOccur)
          {
            $self->prefixError(errorString => $dbObject->errorMessage);
            return undef;
          }
          # we found it
          $accountName = $accountObj->name;
        }
        my $stockObj = $dbObject->getSubAccountInfo(number => $subAccount);
        my $subAccountName = "";
        if (!defined $stockObj)
        {
          if ($dbObject->didErrorOccur)
          {
            $self->prefixError(errorString => $dbObject->errorMessage);
            return undef;
          }
          # we didn't find it
          $self->prefixError(errorString => "Stock ($subAccount) was not found!");
          return undef;
        }
        else
        {
          if ($dbObject->didErrorOccur)
          {
            $self->prefixError(errorString => $dbObject->errorMessage);
            return undef;
          }
          # we found it
          $subAccountName = $stockObj->name;
        }
        $doc->print("<p><font color=\"red\">There were no lots found for this Account ($accountName) and Security ($subAccountName) and Ledger ($ledgerCode) and Date ($date).</font></p>");
        $doc->print("<form>\n");
        $doc->print("<input type=\"button\" value=\"Close\" onClick=\"self.close();\">\n");
        $doc->print("</form>\n");
      }
      else
      {
        $doc->print("<p>Enter the Number of Shares to pull from each lot.</p>");
        $doc->print("<form name=\"lot_form\">\n");
        $doc->print("<table width=\"100%\">\n");
        $doc->print("<tr>\n");
        $doc->print("  <th align=\"left\">Lot&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>");
        $doc->print("  <th align=\"left\">Date&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>");
        $doc->print("  <th align=\"right\">Shares Avail</th>");
        $doc->print("  <th>&nbsp;</th>");
        $doc->print("  <th align=\"right\">&nbsp;&nbsp;&nbsp;&nbsp;Cost</th>");
        $doc->print("  <th>&nbsp;</th>\n");
        $doc->print("  <th align=\"right\">&nbsp;&nbsp;&nbsp;&nbsp;Cost/Share</th>");
        $doc->print("  <th>&nbsp;&nbsp;</th>");
        $doc->print("  <th align=\"left\">Shares</th>");
        $doc->print("  </tr>\n");
        $doc->setFocus("javascript");
        $doc->print("var shareArray = new Array();\n");
        my $costPerShare;
        my $totalShares = 0;
        my $sharesTotalCost = 0;
        foreach my $index (@lotNodes)
        {
          if ($index->{shares} != 0.0) #dont print this row if the share balance is 0
          {
            $doc->setFocus("body");
            $doc->print("<tr>\n");
            $doc->print("  <td align=\"left\">$index->{lot}</td>\n");
            $doc->print("  <td align=\"left\">$index->{eventDate}</td>\n");
            $doc->print("  <td align=\"right\">$index->{shares}</td>\n");
            $doc->print("  <td>&nbsp;</td>\n");
            $doc->print("  <td align=\"right\">$index->{cost}</td>\n");
            $doc->print("  <td>&nbsp;</td>\n");
            $costPerShare = $index->{cost} / $index->{shares}; #/
            $doc->print("  <td align=\"right\">$costPerShare</td>\n");
            $doc->print("  <td>&nbsp;</td>\n");
            $doc->print("  <td><input type=\"text\" name=\"l$index->{lot}\" size=\"10\" value=\"0.0\" onChange=\"checkMax(this); updateTotal();\"></td>\n");
            $doc->print("  </tr>\n");
            $doc->setFocus("javascript");
            $doc->print("shareArray[\"$index->{lot}\"]=$index->{shares};\n");
            $totalShares += $index->{shares};
            $sharesTotalCost += $index->{cost};
          }
        }
        $doc->setFocus("body");
        $doc->print("<tr>\n");
        $doc->print("  <th colspan=\"2\" align=\"right\">Total</th>\n");
        $doc->print("  <td align=\"right\"><b>$totalShares</b></th>\n");
        $doc->print("  <td>&nbsp;</th>\n");
        $doc->print("  <td align=\"right\"><b>$sharesTotalCost</b></th>\n");
        $doc->print("  <td>&nbsp;</td>\n");
        $doc->print("  <td>&nbsp;</td>\n");
        $doc->print("  <td>&nbsp;</td>\n");
        $doc->print("  <td><input type=\"text\" name=\"total\" size=\"10\" onFocus=\"this.blur();\"></td>\n");
        $doc->print("</tr>\n");
        $doc->print("</table>\n");
        $doc->print("<table width=\"100%\">\n");
        $doc->print("  <tr>\n");
        $doc->print("    <td>\n");
        $doc->print("<input type=\"button\" value=\"All\" onClick=\"selectAll();\">\n");
        $doc->print("    </td>\n");
        $doc->print("    <td>\n");
        $doc->print("<input type=\"button\" value=\"Done\" onClick=\"writeLots();self.close();\">\n");
        $doc->print("    </td>\n");
        $doc->print("  </tr>\n");
        $doc->print("</table>\n");
      }
      $doc->setFocus("body");
      $doc->print("</form>");
      if (!$display->javascript_floatify(doc => $doc) )
      {
         $self->prefixError(errorString => "Javascript Error<br>\n$display->errorMessage()");
         return undef;
      }
      $doc->setFocus("javascript");
      $doc->print("function checkMax(textfield)\n");
      $doc->print("{\n");
      $doc->print("  textfield.value = floatify(textfield.value, 6);\n");
      $doc->print("  var name = textfield.name;\n");
      $doc->print("  name = name.substr(1);\n");
      $doc->print("  if (textfield.value > shareArray[name])\n");
      $doc->print("  {\n");
      $doc->print("    alert(\"Shares chosen cannot be more than what is available!\");\n");
      $doc->print("    textfield.value = floatify(shareArray[name], 6);\n");
      $doc->print("  }\n");
      $doc->print("  if (textfield.value < 0)\n");
      $doc->print("  {\n");
      $doc->print("    alert(\"Shares chosen cannot be less than 0!\");\n");
      $doc->print("    textfield.value = '0.0';\n");
      $doc->print("    textfield.value = floatify(textfield.value, 6);\n");
      $doc->print("  }\n");
      $doc->print("}\n");
      $doc->print("function writeLots()\n");
      $doc->print("{//write data back into parent window into mainForm.lots\n");
      $doc->print("  var lotstring ='';\n");
      $doc->print("  var total = 0.0;\n");
      $doc->print("  var count = 0;\n");
      $doc->print("  for (var i in shareArray)\n");
      $doc->print("  {\n");
      $doc->print("    var val = 0.0;\n");
      $doc->print("    val = eval(\"document.lot_form.l\"+i+\".value;\");\n");
      $doc->print("    if (val != '' && val > 0)\n");
      $doc->print("    {\n");
      $doc->print("      total = eval(total + ' + ' + val + ';');\n");
      $doc->print("      if (count > 0)\n");
      $doc->print("      {\n");
      $doc->print("        lotstring += '|';\n");
      $doc->print("      }\n");
      $doc->print("      lotstring += i + '=' + val;\n");
      $doc->print("      count++;\n"); # increment count only for valid entries
      $doc->print("    }\n");
      $doc->print("  }\n");
      $doc->print("  total = floatify(total, 6);\n");
      $doc->print("  var windowref = window.opener;\nwindowref.document.mainForm.lots.value = lotstring;\nwindowref.document.mainForm.shares.value = total;\n");
      $doc->print("}\n");
      $doc->print("function updateTotal()\n");
      $doc->print("{\n");
      $doc->print("  var total = 0.0;\n");
      $doc->print("  var val = 0.0;\n");
      $doc->print("  var evalme;\n");
      $doc->print("  var evalstring;\n");
      $doc->print("  for (var i in shareArray)\n");
      $doc->print("  {\n");
      $doc->print("    evalme = \"document.lot_form.l\"+i+\".value;\";\n");
      $doc->print("    total += parseFloat(eval(evalme));\n");
      $doc->print("  }\n");
      $doc->print("  total = floatify(total, 6);\n");
      $doc->print("  document.lot_form.total.value = total;\n");
      $doc->print("}\n\n");
      $doc->print("function selectAll()\n");
      $doc->print("{\n");
      $doc->print("  for (var i in shareArray)\n");
      $doc->print("  {\n");
      $doc->print("    eval(\"document.lot_form.l\"+i+\".value = shareArray[\\\"\"+i+\"\\\"]\")\n");
      $doc->print("  }\n");
      $doc->print("}\n\n");
    }
    return $doc;
}

sub actionDeref
{
  my $self = shift;
  my %args =  ( @_ );
  my $name = "Error!";
  my $action = "NoAction";
  if (defined $args{action})
  {
    $action = $args{action};
  }
  else
  {
    return $action;
  }

  my %actionhash = %{$self->{variables}->{investmentActionHash}};

  $name = $actionhash{$action};

  return $name;
}

#pre: action => "event_code"
#returns: -1 for failure, a number corresponding to an event
#sum: takes an event code and changes it to a number
#ex: buy -> 10
sub actionDeref2Number
{
  my $self = shift;
  my %args =  ( @_ );
  my $action = "";
  my $number = "-1";
  if (defined $args{action})
  {
    $action = $args{action};
  }
  else
  {
    return $action;
  }
  my %numberhash = %{$self->{variables}->{investmentNumberHash}};
  $number = $numberhash{$action};
  return $number;
}

# displayEvents
# requires:
# optional:
# returns: HTMLObject::Normal document or undef on error
sub displayEvents
{
  my $self = shift;
  my $phrases = $self->{langObj};
  my $doc = HTMLObject::Normal->new;
  my %input = ( @_ );
  my %args = (accounts => "", stocks => "", actions => "", numEntriesToDisplay => 20, showEvents => "global+local");
  $args{dbObject} = $input{dbObject};
  my $errStr = "Portal::Accounting::Investments::Journal->displayEvents()  - Error!<br>\n";
  my $display = Portal::Accounting::DisplayItems->new(langObj => $self->{langObj});
  my $state = "Investments::Journal";
  my $uriState = $doc->encodeString(string => "Investments::Journal");
  my $otherArgs = "";
  my $altArgs = "";
  my $siteDir = $self->{configObj}->{siteDir};
  my $ledgerCode;
  # validate input from $self->{input} and then gather the data and display it.
  $args{accounts} = $self->{input}->{accounts};
  $args{stocks} = $self->{input}->{stocks};
  $args{actions} = $self->{input}->{actions};
  $args{startDate} = $self->{input}->{startDate};
  $args{endDate} = $self->{input}->{endDate};
  $args{numEntriesToDisplay} = $self->{input}->{numEntriesToDisplay};
  $args{showEvents} = $self->{input}->{showEvents};
  $args{startIndex} = $self->{input}->{startIndex};
  $args{ledger} = $self->{input}->{ledger};
  $args{ledgerCode} = $self->{input}->{ledgerCode};
  $ledgerCode = $self->{input}->{ledgerCode};

  if ($args{ledgerCode} eq "")
  {
    $self->prefixError("ledgerCode is ''!<br>\n");
    return undef;
  }
  my $printable = (exists $self->{input}->{printable} ? "true" : "false");  # are we displaying for printing?
  $args{errorStr} = "";

  if ((!exists $input{dbObject}) || (!defined $input{dbObject}))
  {
    $self->prefixError("dbObject not defined or invalid!<br>\n");
    return undef;
  }
  if ($args{dbObject}->didErrorOccur)
  {
    $self->prefixError($args{dbObject}->errorMessage);
    return undef;
  }
  $self->{input}->{codeFontSize} = (exists $self->{input}->{codeFontSize} ? $self->{input}->{codeFontSize} : "-1");
  $self->{input}->{normalFontSize} = (exists $self->{input}->{normalFontSize} ? $self->{input}->{normalFontSize} : "+0");

  # do quick validation
  foreach my $arg (split /\0/, $args{accounts})
  {
    $otherArgs .= "&accounts=$arg";
    $altArgs .= (length $altArgs > 0 ? "        " : "") . "<input type=\"hidden\" name=\"accounts\" value=\"$arg\">\n";
  }
  $args{accounts} =~ s/\0/,/g;
  if (length $args{accounts} > 0 && $args{accounts} !~ /^\d+(,\d+)*$/)
  {
    $args{errorStr} .= "accounts = '$args{accounts}' is invalid!<br>\n";
    $args{accounts} = "";
    $doc = $self->display(%args);
    if ($self->didErrorOccur)
    {
      $self->prefixError(errorString => $args{errStr} . $self->errorMessage());
      return undef;
    }
    return $doc;
  }
  foreach my $arg (split /\0/, $args{stocks})
  {
    $otherArgs .= "&stocks=$arg";
    $altArgs .= (length $altArgs > 0 ? "        " : "") . "<input type=\"hidden\" name=\"stocks\" value=\"$arg\">\n";
  }
  $args{stocks} =~ s/\0/,/g;
  if (length $args{stocks} > 0 && $args{stocks} !~ /^\d+(,\d+)*$/)
  {
    $args{errorStr} .= "stocks = '$args{stocks}' is invalid!<br>\n";
    $args{stocks} = "";
    $doc = $self->display(%args);
    if ($self->didErrorOccur)
    {
      $self->prefixError(errorString => $args{errStr} . $self->errorMessage());
      return undef;
    }
    return $doc;
  }
  foreach my $arg (split /\0/, $args{actions})
  {
    $otherArgs .= "&actions=$arg";
    $altArgs .= (length $altArgs > 0 ? "        " : "") . "<input type=\"hidden\" name=\"actions\" value=\"$arg\">\n";
  }
  $args{actions} =~ s/\0/,/g;
  if (length $args{actions} > 0 && $args{actions} !~ /^(\d+|memo)(,(\d+|memo))*$/)
  {
    $args{errorStr} .= "actions = '$args{actions}' is invalid!<br>\n";
    $args{actions} = "";
    $doc = $self->display(%args);
    if ($self->didErrorOccur)
    {
      $self->prefixError(errorString => $args{errStr} . $self->errorMessage());
      return undef;
    }
    return $doc;
  }
  $otherArgs .= "&numEntriesToDisplay=$args{numEntriesToDisplay}";
  $altArgs .= (length $altArgs > 0 ? "        " : "") . "<input type=\"hidden\" name=\"numEntriesToDisplay\" value=\"$args{numEntriesToDisplay}\">\n";
  if ($args{numEntriesToDisplay} < 5 || $args{numEntriesToDisplay} > 50 || $args{numEntriesToDisplay} !~ /^\d+$/)
  {
    $args{errorStr} .= "numEntriesToDisplay = '$args{numEntriesToDisplay}' is invalid!<br>\n";
    $args{numEntriesToDisplay} = 20;
    $doc = $self->display(%args);
    if ($self->didErrorOccur)
    {
      $self->prefixError(errorString => $args{errStr} . $self->errorMessage());
      return undef;
    }
    return $doc;
  }
  $otherArgs .= "&showEvents=" . $doc->encodeString(string => $args{showEvents});
  $altArgs .= (length $altArgs > 0 ? "        " : "") . "<input type=\"hidden\" name=\"showEvents\" value=\"$args{showEvents}\">\n";
  if ($args{showEvents} !~ /^(global|global\+local|local)$/)
  {
    $args{errorStr} .= "showEvents = '$args{showEvents}' is invalid!<br>\n";
    $args{showEvents} = "global";
    $doc = $self->display(%args);
    if ($self->didErrorOccur)
    {
      $self->prefixError(errorString => $args{errStr} . $self->errorMessage());
      return undef;
    }
    return $doc;
  }
  $otherArgs .= "&startDate=$args{startDate}&endDate=$args{endDate}";
  $altArgs .= (length $altArgs > 0 ? "        " : "") . "<input type=\"hidden\" name=\"startDate\" value=\"$args{startDate}\">\n        <input type=\"hidden\" name=\"endDate\" value=\"$args{endDate}\">\n";
  my $startDate = $args{startDate};
  my $endDate = $args{endDate};
  if (length $startDate > 0)
  {
    my @dateResult = $self->{methods}->fixupDate(date => $startDate);
    if (!$dateResult[0])
    {
      $args{errorStr} .= "startDate = '$startDate' is invalid!<br>\n";
      $args{startDate} = "";
      $doc = $self->display(%args);
      if ($self->didErrorOccur)
      {
        $self->prefixError();
        return undef;
      }
      return $doc;
    }
    else
    {
      # make sure the date is "fixed".
      $startDate = $dateResult[1];
    }
  }
  if (length $endDate > 0)
  {
    my @dateResult = $self->{methods}->fixupDate(date => $endDate);
    if (!$dateResult[0])
    {
      $args{errorStr} .= "endDate = '$endDate' is invalid!<br>\n";
      $args{endDate} = "";
      $doc = $self->display(%args);
      if ($self->didErrorOccur)
      {
        $self->prefixError();
        return undef;
      }
      return $doc;
    }
    else
    {
      # make sure the date is "fixed".
      $endDate = $dateResult[1];
    }
  }
  #$otherArgs .= "&startIndex=$args{startIndex}";
  if ($args{startIndex} !~ /^\d+$/ || $args{startIndex} < 1)
  {
    $args{errorStr} .= "startIndex = '$args{startIndex}' is invalid!<br>\n";
    $args{startIndex} = 1;
    $doc = $self->display(%args);
    if ($self->didErrorOccur)
    {
      $self->prefixError(errorString => $errStr . $self->errorMessage());
      return undef;
    }
    return $doc;
  }

  # gather the data
  my $securityObj = Portal::Accounting::DB::Security->new(langObj => $self->{langObj}, dbHandle => $self->{dbHandle});
  if ($securityObj->didErrorOccur)
  {
    $args{errorStr} .= $errStr . $securityObj->errorMessage;
    $doc = $self->display(%args);
    if ($self->didErrorOccur)
    {
      $self->prefixError(errorString => $errStr . $self->errorMessage());
      return undef;
    }
    return $doc;
  }

  my $investmentJournalViewObj = $securityObj->getSecurityEventsForDisplay(%args);
  if ($securityObj->didErrorOccur)
  {
    $args{errorStr} .= $errStr . $securityObj->errorMessage;
    $doc = $self->display(%args);
    if ($self->didErrorOccur)
    {
      $self->prefixError(errorString => $errStr . $self->errorMessage());
      return undef;
    }
    return $doc;
  }

  if (!$investmentJournalViewObj->isValid)
  {
    $args{errorStr} .= $errStr . $investmentJournalViewObj->errorMessage;
    $doc = $self->display(%args);
    if ($self->didErrorOccur)
    {
      $self->prefixError(errorString => $errStr . $self->errorMessage());
      return undef;
    }
    return $doc;
  }

  $doc->setTitle("Investment Journal");
  $doc->setFocus("body");
  $doc->setLinkDecorations;
  my $ledgerURI = $doc->encodeString(string => $args{ledger});
  my $codeFontSize = $self->{input}->{codeFontSize};
  my $codeFontSizeURI = $doc->encodeString(string => $codeFontSize);
  my $normalFontSize = $self->{input}->{normalFontSize};
  my $normalFontSizeURI = $doc->encodeString(string => $normalFontSize);
  if ($printable eq "false")
  {
    $doc->setFocus("javascript");
    $doc->print(<<"END_OF_CODE");

function confirmDelete(refNum)
{
  return window.confirm("Do you want to Delete Investment Event with RefNum = '"+refNum+"'?\\nPress OK to Delete, Cancel otherwise.");
}

function changeFont()
{
  document.fontForm.submit();
}

END_OF_CODE

    $doc->setFocus("body");
    $doc->print("<center><h3>Investment Journal</h3></center>\n");

    my $startIndex = $args{startIndex};
    my $InvJ = $phrases->map("Investment Journal");
    $doc->print(<<"END_OF_CODE");
<table border="0" width="100%" cellpadding="0" cellspacing="0">
  <tr>
    <td align="left"><a href="$self->{getInfo}&command=display" onMouseOver="window.status='$InvJ'"><font size="-1">$InvJ</font></a>&nbsp;|
      <a href="$self->{getInfo}&command=displayEvents$otherArgs&startIndex=$startIndex&printable=true&codeFontSize=$codeFontSizeURI&normalFontSize=$normalFontSizeURI" onMouseOver="window.status='Print Page'"><font size="-1">Print Page</font></a>&nbsp;|
      <a href="$self->{getInfo}&command=inputScreen" onMouseOver="window.status='Create New Investment Transaction'"><font size="-1">New Transaction</font></a>
    </td>
    <td align="center">
      <form method="post" action="$siteDir/cgi-bin/index.cgi" name="fontForm">
$self->{postInfo}
        <input type="hidden" name="command" value="displayEvents">
        <input type="hidden" name="startIndex" value="$startIndex">
        $altArgs
        <font size="-1">Change Font Sizes:<br>
        Code <select name="codeFontSize" onChange="changeFont(); return true;">
END_OF_CODE
    for (my $i=2; $i > -3; $i--)
    {
      my $selected = ($i > -1 ? ("+$i" eq $codeFontSize ? " selected" : "") : ("$i" eq $codeFontSize ? " selected" : ""));
      $doc->print("          <option value=\"" . ($i > -1 ? "+" : "") . "$i\"$selected>" . ($i > -1 ? "+" : "") . "$i\n");
    }
    $doc->print(<<"END_OF_CODE");
        </select>&nbsp;&nbsp;Normal <select name="normalFontSize" onChange="changeFont(); return true;">
END_OF_CODE
    for (my $i=2; $i > -3; $i--)
    {
      my $selected = ($i > -1 ? ("+$i" eq $normalFontSize ? " selected" : "") : ("$i" eq $normalFontSize ? " selected" : ""));
      $doc->print("          <option value=\"" . ($i > -1 ? "+" : "") . "$i\"$selected>" . ($i > -1 ? "+" : "") . "$i\n");
    }
    $doc->print(<<"END_OF_CODE");
        </select>
      </form>
    </td>
    <td align="right">
END_OF_CODE
    # now print the Prev, Curr, Next links and the select dropdown to jump around quicker with.
    my $length2 = $args{numEntriesToDisplay} * 2;
    if ($args{startIndex} > $args{numEntriesToDisplay}) # we need a previous link
    {
      my $displayString = "Prev (" . ($args{startIndex}-$args{numEntriesToDisplay}) . " - " . ($args{startIndex}-1) . ")";
      my $startIndex = $args{startIndex}-$args{numEntriesToDisplay};
      $doc->print(<<"END_OF_CODE");
      <a href="$self->{getInfo}&command=displayEvents$otherArgs&startIndex=$startIndex&codeFontSize=$codeFontSizeURI&normalFontSize=$normalFontSizeURI" onMouseOver="window.status='$displayString'"><font size="-1">$displayString</font></a>&nbsp;|&nbsp;
END_OF_CODE
    }

    # now print the current link
    my $displayString = "Curr (" . ($args{startIndex}) . " - " . ($investmentJournalViewObj->{totalEntries} > $args{startIndex}+$args{numEntriesToDisplay}-1 ? ($args{startIndex}+$args{numEntriesToDisplay}-1) : $investmentJournalViewObj->{totalEntries}) . ")";
    my $startIndex = $args{startIndex};
    if ($investmentJournalViewObj->{totalEntries} > 0)
    {
      $doc->print(<<"END_OF_CODE");
      <a href="$self->{getInfo}&command=displayEvents$otherArgs&startIndex=$startIndex&codeFontSize=$codeFontSizeURI&normalFontSize=$normalFontSizeURI" onMouseOver="window.status='$displayString'"><font size="-1">$displayString</font></a>
END_OF_CODE
    }

    if ($args{startIndex}+$args{numEntriesToDisplay} <= $investmentJournalViewObj->{totalEntries})  # we need the next link
    {
      my $displayString = "Next (" . ($args{startIndex}+$args{numEntriesToDisplay}) . " - " . ($investmentJournalViewObj->{totalEntries} > $args{startIndex}+$length2-1 ? ($args{startIndex}+$length2-1) : $investmentJournalViewObj->{totalEntries}) . ")";
      my $startIndex = $args{startIndex}+$args{numEntriesToDisplay};
      $doc->print(<<"END_OF_CODE");
      &nbsp;|&nbsp;<a href="$self->{getInfo}&command=displayEvents$otherArgs&startIndex=$startIndex&codeFontSize=$codeFontSizeURI&normalFontSize=$normalFontSizeURI" onMouseOver="window.status='$displayString'"><font size="-1">$displayString</font></a>
END_OF_CODE
    }
    my $header = ($investmentJournalViewObj->{totalEntries} > 0 ? "&nbsp;|&nbsp;&nbsp;" : "");
    $doc->print(<<"END_OF_CODE");
      $header<font size="-1"><b>$investmentJournalViewObj->{totalEntries}</b> Total, showing <b>$args{numEntriesToDisplay}</b>/page.</font>
    </td>
  </tr>
</table>
<br>
END_OF_CODE
  }
  else
  {
    # display the Investment Journal header and make it the link to return to the current page (non-printable) view.
    my $startIndex = $args{startIndex};
    $doc->print(<<"END_OF_CODE");
<center>
<a href="$self->{getInfo}&command=displayEvents$otherArgs&startIndex=$startIndex&codeFontSize=$codeFontSizeURI&normalFontSize=$normalFontSizeURI" onMouseOver="window.status='Return to Investment Journal View'"><font color="black"><h2>Investment Journal</h2></font></a>
</center>
<br>
END_OF_CODE
  }

  # now display the Legend
  my %legend = ( L => "Ledger Code,<br>*=Global Event",
  	         S => "Security",
  	         S2 => "Security2",
  	         A => "Account",
  	         NA => "New Account",
  	         OA => "Old Account",
  	         P => "Price",
  	         Am => "Amount",
  	         Sh => "Shares",
  	         OS => "Old Shares",
  	         NS => "New Shares",
  	         C => "Commission",
  	         Lo => "Lot",
  	         NL => "New Lot",
  	         OL => "Old Lot",
  	         "C%" => "Cost Percentage"
  	       );

  my %accounts;
  my %stocks;

  # lookup all stocks Numbers => Names and accounts Numbers => Names and store in hashes to pass into the securityEvent Object display method.
  my @list = $args{dbObject}->getListOfAccounts(location => "local", type => 3, ledgerCode => $args{ledgerCode});
  if ($args{dbObject}->didErrorOccur)
  {
    $self->prefixError(errorString => $errStr . $args{dbObject}->errorMessage);
    return undef;
  }
  # now make entries in the accounts hash for every account returned.
  foreach my $acctObj (@list)
  {
    $accounts{$acctObj->{number}} = $doc->formEncodeString(string => $acctObj->name);
  }

  my @list = $args{dbObject}->getListOfSubAccounts(type => "7,70", orderBy => 'sub_acc_name');
  if ($args{dbObject}->didErrorOccur)
  {
    $self->prefixError(errorString => $errStr . $args{dbObject}->errorMessage);
    return undef;
  }
  # now make entries in the stocks hash for every sub account returned.
  foreach my $stockObj (@list)
  {
    $stocks{$stockObj->{number}} = $doc->formEncodeString(string => $stockObj->name);
  }
  
  # get the start date of the current fiscal year and also find out if we are in the first fiscal year or not.
  my $currFiscalYearObj = $args{dbObject}->getCurrentFiscalYear(ledgerCode => $args{ledgerCode});
  if ($args{dbObject}->didErrorOccur)
  {
    $self->prefixError(errorString => $errStr . $args{dbObject}->errorMessage);
    return undef;
  }
  if (! defined $currFiscalYearObj)
  {
    $self->prefixError(errorString => $errStr . "Current Fiscal Year info not returned!<br>\n");
    return undef;
  }
  my $fyStart = $currFiscalYearObj->startDate;

  my @knownFiscalYears = $args{dbObject}->getAllKnownYears(ledgerCode => $args{ledgerCode});
  if ($args{dbObject}->didErrorOccur)
  {
    $self->prefixError(errorString => $errStr . $args{dbObject}->errorMessage);
    return undef;
  }
  if (! defined @knownFiscalYears)
  {
    $self->prefixError(errorString => $errStr . "All Known Fiscal Years not returned!<br>\n");
    return undef;
  }
  my $firstYearObj = $knownFiscalYears[0];
  my $inFirstYear = ($firstYearObj->year == $currFiscalYearObj->year ? 1 : 0);

  # now walk over the returned data and display the Investment Journal stuff.
  $doc->print(<<"END_OF_CODE");
<form method="post" action="$siteDir/cgi-bin/index.cgi" name="investViewForm">
  $self->{postInfo}
  <input type="hidden" name="command" value="deleteEntry">
  <input type="hidden" name="refNum" value="">
  <input type="hidden" name="startIndex" value="$args{startIndex}">
  $altArgs
  <input type="hidden" name="codeFontSize" value="$codeFontSize">
  <input type="hidden" name="normalFontSize" value="$normalFontSize">
<table border="0" cellpadding="0" cellspacing="0" width="100%">
END_OF_CODE

  # Sell grouping data variables.
  my $priceTotal = "0.00";
  my $sharesTotal = "0.00";
  my $commissionTotal = "0.00";
  my $masterEventRef = "-1";
  my $startColor = "lightgrey";
  my $numSells = 0;
  my $count = 0;
  my $lastColor = "white";

  for ($count=0; $count < scalar @{$investmentJournalViewObj->{securityEvents}}; $count++)
  {
    my $color = ($lastColor eq "white" ? "lightgrey" : "white");
    # see if we just closed out a sell run that needs to be totaled.
    if ($count > 0 && $investmentJournalViewObj->{securityEvents}[$count-1]->{eventCode} == 20 && $numSells > 1 && ($investmentJournalViewObj->{securityEvents}[$count]->{eventCode} ne "sell" || ($investmentJournalViewObj->{securityEvents}[$count]->{eventCode} == 20 && $investmentJournalViewObj->{securityEvents}[$count]->{masterEventRef} != $masterEventRef)))
    {
      $color = $startColor;
      $doc->print("  <tr bgcolor=\"$color\">\n");
      $doc->print("    <td width=\"" . ($printable eq "false" ? "95%" : "100%") . "\">\n");
      # output the totals.
      $doc->print("      <table border=\"1\" cellpadding=\"1\" cellspacing=\"0\" width=\"100%\">\n");
      $doc->print("        <tr>\n");
      $doc->print("          <td align=\"right\" width=\"44%\"><b>Totals:</b></td>\n");
      $doc->print("          <td align=\"right\" width=\"14%\"><i>" . $display->commify(value => sprintf("%.2f", $priceTotal)) . "</i></td>\n");
      $doc->print("          <td align=\"right\" width=\"14%\"><i>" . $display->commify(value => sprintf("%.2f", $sharesTotal)) . "</i></td>\n");
      $doc->print("          <td align=\"right\" width=\"15%\"><i>" . $display->commify(value => sprintf("%.2f", $commissionTotal)) . "</i></td>\n");
      $doc->print("          <td align=\"right\" width=\"13%\">&nbsp;</td>\n");
      $doc->print("        </tr>\n");
      $doc->print("      </table>\n");
      $doc->print("    </td>\n");
      if ($printable eq "false")
      {
        $doc->print("    <td width=\"5%\" align=\"center\">&nbsp;</td>\n");
      }
      $doc->print("  </tr>\n");

      # reset variables.
      $priceTotal = "0.00";
      $sharesTotal = "0.00";
      $commissionTotal = "0.00";
      $masterEventRef = "-1";
      $numSells = 0;
      $lastColor = $startColor;
    }
    $color = ($lastColor eq "white" ? "lightgrey" : "white");  # recalculate.
    if ($investmentJournalViewObj->{securityEvents}[$count]->{eventCode} == 20 && $numSells > 0 && $investmentJournalViewObj->{securityEvents}[$count]->{masterEventRef} == $masterEventRef)
    {
      $color = $startColor;
    }
    $doc->print("  <tr bgcolor=\"$color\">\n");
    $doc->print("    <td width=\"" . ($printable eq "false" ? "95%" : "100%") . "\">\n");
    my $output = $investmentJournalViewObj->{securityEvents}[$count]->display(printable => $printable, 
                 stocks => \%stocks, accounts => \%accounts, displayObj => $display, codeFontSize => $codeFontSize, normalFontSize => $normalFontSize);
    if ($investmentJournalViewObj->{securityEvents}[$count]->didErrorOccur)
    {
      $args{errorStr} .= $errStr . $investmentJournalViewObj->{securityEvents}[$count]->errorMessage;
      $doc = $self->display(%args);
      if ($self->didErrorOccur)
      {
        $self->prefixError(errorString => $errStr . $self->errorMessage());
        return undef;
      }
      return $doc;
    }
    $output =~ s/^(.+)$/      $1/gm;
    $doc->print($output);
    $doc->print("    </td>\n");
    if ($printable eq "false")
    {
      $doc->print("    <td width=\"5%\" align=\"center\">\n");
      # check and see if they can delete the entry
      if ($self->transactionDeletable(calcDate => $investmentJournalViewObj->{securityEvents}[$count]->calcDate, fyStart => $fyStart, inFirstYear => $inFirstYear))
      {
        $doc->print("      <font size=\"-2\"><input type=\"button\" name=\"delete_$count\" value=\"D\" onclick=\"if (confirmDelete($investmentJournalViewObj->{securityEvents}[$count]->{eventRefNum})) { document.investViewForm.refNum.value=$investmentJournalViewObj->{securityEvents}[$count]->{eventRefNum}; document.investViewForm.submit(); }\"></font>\n");
      }
      else
      {
        $doc->print("      &nbsp;\n");
      }
      $doc->print("    </td>\n");
    }
    $doc->print("  </tr>\n");
    
    # see if we just started a sell run or if we are in one and need to update the values.
    if ($investmentJournalViewObj->{securityEvents}[$count]->{eventCode} == 20)
    {
      my $obj = $investmentJournalViewObj->{securityEvents}[$count];
      if ($numSells > 0 && $obj->{masterEventRef} == $masterEventRef)  # update
      {
        $numSells++;
        $priceTotal += $obj->{price};
        $sharesTotal += $obj->{shares};
        $commissionTotal += $obj->{commission};
      }
      else  # start anew.
      {
        $numSells=1;
        $masterEventRef = $obj->{masterEventRef};
        $priceTotal = $obj->{price};
        $sharesTotal = $obj->{shares};
        $commissionTotal = $obj->{commission};
        $startColor = $color;
      }
    }
    $lastColor = $color;
  }
  
  # we have to check for / handle the case where the last item was a sell and it was part of a run.
  if ($count > 0 && $investmentJournalViewObj->{securityEvents}[$count-1]->{eventCode} == 20 && $numSells > 1)
  {
    my $color = $startColor;
    $doc->print("  <tr bgcolor=\"$color\">\n");
    $doc->print("    <td width=\"" . ($printable eq "false" ? "95%" : "100%") . "\">\n");
    # output the totals.
    $doc->print("      <table border=\"1\" cellpadding=\"1\" cellspacing=\"0\" width=\"100%\">\n");
    $doc->print("        <tr>\n");
    $doc->print("          <td align=\"right\" width=\"44%\"><b>Totals:</b></td>\n");
    $doc->print("          <td align=\"right\" width=\"14%\"><i>" . $display->commify(value => sprintf("%.2f", $priceTotal)) . "</i></td>\n");
    $doc->print("          <td align=\"right\" width=\"14%\"><i>" . $display->commify(value => sprintf("%.2f", $sharesTotal)) . "</i></td>\n");
    $doc->print("          <td align=\"right\" width=\"15%\"><i>" . $display->commify(value => sprintf("%.2f", $commissionTotal)) . "</i></td>\n");
    $doc->print("          <td align=\"right\" width=\"13%\">&nbsp;</td>\n");
    $doc->print("        </tr>\n");
    $doc->print("      </table>\n");
    $doc->print("    </td>\n");
    if ($printable eq "false")
    {
      $doc->print("    <td width=\"5%\" align=\"center\">&nbsp;</td>\n");
    }
    $doc->print("  </tr>\n");
  }

  $doc->print(<<"END_OF_CODE");
</table>
</form>
END_OF_CODE

  # display the Legend info at the bottom of the page.
  $doc->print("<center>");
  $doc->print("<table align=\"left\" border=\"1\" cellpadding=\"1\" cellspacing=\"0\">\n");

  my $legendMax = 4;      # the max number of code to descriptions to display per row.
  my $legendCodeMax = 3;  # the max number of codes to display per row
  my $count = 0;
  foreach my $item (sort keys %legend)
  {
    if ($count == 0)
    {
      $doc->print("  <tr>\n");
    }
    $doc->print("    <td align=\"left\">\n");
    $doc->print("      <font size=\"-1\"><b>$item</b>=$legend{$item}</font>\n");
    $doc->print("    </td>\n");
    $count++;
    if ($count == $legendMax)
    {
      $doc->print("  </tr>\n");
      $count = 0;
    }
  }
  if ($count != 0)
  {
    my $temp = $legendMax - $count;
    for ($count=0; $count < $temp; $count++)
    {
      $doc->print("    <td>&nbsp;</td>\n");
    }
    $doc->print("  </tr>\n");
  }
  
  $doc->print("</table>\n"); 

  # now display the list of action codes and their full names (6 to a line)
  my $count = 0;
  $doc->print("<table border=\"1\" cellpadding=\"1\" cellspacing=\"0\">\n");
  foreach my $code (sort keys %{$self->{variables}->{investmentActionHash}})
  {
    if ($count == 0)
    {
      $doc->print("  <tr>\n");
    }
    $doc->print("    <td align=\"left\">\n");
    $doc->print("      <font size=\"-1\"><b>$code</b>=" . $self->{variables}->{investmentActionHash}->{$code} . "</font>");
    $doc->print("    </td>\n");
    $count++;
    if ($count == $legendCodeMax)
    {
      $doc->print("  </tr>\n");
      $count = 0;
    }
  }
  if ($count != 0)
  {
    my $temp = $legendCodeMax - $count;
    for ($count=0; $count < $temp; $count++)
    {
      $doc->print("    <td>&nbsp;</td>\n");
    }
    $doc->print("  </tr>\n");
  }
  $doc->print("</table>\n</center>\n<br>\n<br>\n");

  if ($printable eq "true")
  {
    $doc->setOnload(code => "window.print()");
  }

  return $doc;
}

# transactionDeletable
# requires: calcDate, fyStart, inFirstYear
# optional:
# returns: 1 or 0
# summary: This routine determines if the specified date is a deletable date.
# If we are inFirstYear then all dates are deletable.  If not, then the calcDate
# must be >= to fyStart.
sub transactionDeletable
{
  my $self = shift;
  my %args = ( calcDate => "", fyStart => "", inFirstYear => 0, @_ );
  my $calcDate = $args{calcDate};
  my $fyStart = $args{fyStart};
  my $inFirstYear = $args{inFirstYear};
  my $errStr = "Portal::Accounting::Investments::Journal->transactionDeletable() -  Error!<br>\n";
  my $deletable = 0;  # default to not being deletable.
  
  if ($inFirstYear !~ /^(0|1)$/)
  {
    $self->prefixError(errorString => $errStr . "inFirstYear = '$inFirstYear' is invalid!<br>\n");
    return $deletable;
  }
  if ($calcDate !~ /^(\d{2}(-|\/)\d{2}(-|\/)\d{4})$/)
  {
    $self->prefixError(errorString => $errStr . "calcDate = '$calcDate' is invalid!<br>\n");
    return $deletable;
  }
  if ($fyStart !~ /^(\d{2}(-|\/)\d{2}(-|\/)\d{4})$/)
  {
    $self->prefixError(errorString => $errStr . "fyStart = '$fyStart' is invalid!<br>\n");
    return $deletable;
  }
  
  # now convert calcDate to YYYYMMDD format.
  $calcDate =~ m/^(\d{2})(-|\/)(\d{2})(-|\/)(\d{4})$/;
  $calcDate = $5 . $1 . $3;
  
  # now convert fyStart to YYYYMMDD format.
  $fyStart =~ m/^(\d{2})(-|\/)(\d{2})(-|\/)(\d{4})$/;
  $fyStart = $5 . $1 . $3;
  
  if ($inFirstYear)
  {
    $deletable = 1;
  }
  else
  {
    if ($calcDate >= $fyStart)
    {
      $deletable = 1;
    }
  }
  
  return $deletable;
}

# req: lots => a string containing lot=shares|lot=shares|..., eventDate =>
# opt:   securityDBObject--if not sent in the lots will simply be hashified
# ret: scalar, Hash{lot} = shares
#  scalar 0 = sucess, 1 = shares too large, 2 = bad string
sub lotHashifyCheck
{
  my $self = shift;
  my %args = (@_);
  my $securityDBObject = undef;
  if (defined $args{securityDBObject})
  {
    $securityDBObject = $args{securityDBObject};
  }
  my $lots = $args{lots};
  my $date = $args{eventDate};
  #split the lots on | then on =
  my @lotArray = split(/\|/, $lots);
  my %lotHash;
  my ($tempLot, $tempShares, $tempNode);
  foreach my $lotIndex (@lotArray)
  {
    ($tempLot, $tempShares) = split(/\=/, $lotIndex);
    if ($tempLot eq "" || $tempShares eq "")
    {
        return (2, undef);
    }
    else
    {
      $lotHash{$tempLot} = $tempShares;

      #if securityDBOjbect was sent in get the balance for the lot
      if (defined $securityDBObject)
      {
        $tempNode = $securityDBObject->getLotBalance(lot => $tempLot, eventDate => $date);
        if ($securityDBObject->didErrorOccur)
        {
          $self->prefixError("Balance for Lot: $tempLot, $date failed<br>\n". $securityDBObject->errorMessage());
          return (undef, undef);
        }
        #check to make sure the shares chosen are <= balance
        if ($tempShares > $tempNode->{shares})
        {
          return (1, undef);
        }
      }
    }
  }#endfor
  return (0, \%lotHash);
}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut
