# JournalEntry.pm - The Object Class that provides a JournalEntry Object
#$Id: JournalEntry.pm,v 1.1 2004/02/11 21:06:45 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.
# Created by Xperience, Inc.

package Portal::Accounting::JournalEntry;
use strict;
use HTMLObject::Base;
use HTMLObject::Normal;
use Portal::Accounting::Objects::HTMLJournalEntry;
use Portal::Accounting::Base;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Portal::Accounting::Base Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.01';

# global variables.

# modify to represent the commands this state provides.
my %commands = (
  display => "Main Menu",
  newCheck =>,
  editCheck =>,
  saveCheck =>,
  newDeposit =>,
  editDeposit =>,
  saveDeposit =>,
  newDoubleEntry =>,
  editDoubleEntry =>,
  saveDoubleEntry =>,
  );

=head1 NAME

JournalEntry

=head1 SYNOPSIS

  use Portal::Accounting::JournalEntry;
  my $obj = Portal::Accounting::JournalEntry->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }
  $obj->run(command => $command);

=head1 DESCRIPTION

This set of screens will hold the forms for data Entry in the system.

=head1 Exported FUNCTIONS

B<NOTE>: I<bool> = 1(true), 0(false)

=over 4

=item scalar new(codeVersion, release, DBVersion, #!!!portal args??)

 Creates a new instance of the Portal::Accounting::JournalEntry module.
 See Portal::Accounting::Base(3) for a listing of required arguments.

 Instantiates an instance of the JournalEntry State. !!Anything passed in here should be
 data that never changes from one run of a command to another such as database handles
 portalSession? input? cookieObj? dont fit that criterion

=cut

sub new
{
  my $class = shift;
  my $self = $class->SUPER::new(@_);
  my %args = ( release => "", DBVersion => "", @_ );

  $self->{commands} = \%commands;

  if ($self->didErrorOccur)
  {
    $self->prefixError();
    return $self;
  }

  # instantiate anything unique to this module

  # do validation
  if (!$self->Portal::Accounting::JournalEntry::isValid)
  {
    # the error is set in the isValid() method.
    return $self;
  }

  # do anything else you might need to do.

  return $self;
}

=item bool isValid(void)

 Returns 0 or 1 to indicate if the object is valid.
 The error will be available via errorMessage().

=cut

sub isValid
{
  my $self = shift;

  # make sure our Parent class is valid.
  if (!$self->SUPER::isValid())
  {
    $self->prefixError();
    return 0;
  }

  # validate our parameters.

  if ($self->numInvalid() > 0 || $self->numMissing() > 0)
  {
    $self->postfixError($self->genErrorString("all"));
    return 0;
  }

  return 1;
}

=item HTMLObject display()

    Implements the default screen for this state.

=cut

sub c_display
{
  my $self = shift;
  my %args = ( command => "display", message => "", @_ );
  my $command = $args{command};
  my $siteDir = $self->{configObj}->{siteDir};
  my $phrases = $self->{langObj};
  my $message = $args{message};
  my $ledger = $self->{input}->{ledger};
  my $ledgerCode = $self->{input}->{ledgerCode};

  my $doc = HTMLObject::Normal->new;
  if (not defined $doc || $doc->didErrorOccur)
  {
    $self->prefixError("Could not create HTMLObject<br>\n");
    return undef;
  }
  $doc->setTitle($phrases->map("JournalEntry").":".$ledger);
  my $url = $self->{baseURL};

  # include the form_methods.js file
#   $doc = $self->{methods}->includeCoreJSFiles(doc => $doc);
#   if ($self->{methods}->didErrorOccur)
#   {
#     $self->setError($self->{methods}->errorMessage);
#     return undef;
#   }
# 
#   # create the help JS function.
#   $doc = $self->{methods}->displayJSHelper(doc => $doc, type => "help", langObj => $self->{langObj});
#   if ($self->{methods}->didErrorOccur)
#   {
#     $self->setError($self->{methods}->errorMessage);
#     return undef;
#   }
# 
#   # create the validateClose JS function.
#   $doc = $self->{methods}->displayJSHelper(doc => $doc, type => "closeApp", langObj => $self->{langObj});
#   if ($self->{methods}->didErrorOccur)
#   {
#     $self->setError($self->{methods}->errorMessage);
#     return undef;
#   }

  $doc->print(<<"EOF");
<h2>General Ledger Data Entry</h2>
<a href="?app=$self->{input}->{app}&state=Search&command=display&ledger=$self->{input}->{ledger}&ledgerCode=$self->{input}->{ledgerCode}">Search</a><br>
<a href="?app=$self->{input}->{app}&state=$self->{input}->{state}&command=newCheck&ledger=$self->{input}->{ledger}&ledgerCode=$self->{input}->{ledgerCode}">Check</a><br>
<a href="?app=$self->{input}->{app}&state=$self->{input}->{state}&command=newDeposit&ledger=$self->{input}->{ledger}&ledgerCode=$self->{input}->{ledgerCode}">Deposit</a><br>
<a href="?app=$self->{input}->{app}&state=$self->{input}->{state}&command=newDoubleEntry&ledger=$self->{input}->{ledger}&ledgerCode=$self->{input}->{ledgerCode}">DoubleEntry</a><br>
<a href="?app=$self->{input}->{app}&state=$self->{input}->{state}&command=newCreditPurchase&ledger=$self->{input}->{ledger}&ledgerCode=$self->{input}->{ledgerCode}">Credit Purchase</a><br>
<a href="?app=$self->{input}->{app}&state=$self->{input}->{state}&command=newCreditDeposit&ledger=$self->{input}->{ledger}&ledgerCode=$self->{input}->{ledgerCode}">Credit Deposit</a><br>

EOF
  return $doc;
}

sub c_newCheck
{
  my $self = shift;
  return $self->c_editCheck;
}

sub c_saveNewCheck
{
  my $self = shift;
  return $self->c_saveCheck;
}

sub c_editCheck
{
  my $self = shift;
  my %args = ( message => "", @_ );
  my $phrases = $self->{langObj};
  my $doc = HTMLObject::Normal->new;
  if (not defined $doc || $doc->didErrorOccur)
  {
    $self->setError(errorString => "Could not create HTMLObject<br>\n");
    return undef;
  }
  $doc->setTitle($phrases->map("Edit Check").":$self->{input}->{ledger}");
  $doc->setFocus("body");
  my $je = Portal::Accounting::Objects::HTMLJournalEntry->new(langObj => $self->{langObj});
  if ($je->didErrorOccur)
  {
    $doc->print("<h4>Error Occurred while initializing Journal Entry Object</h4>".$je->errorMessage."<br>\n");
  }
  else
  {
    my $empty = "n";
    if ($self->{input}->{command} eq "newCheck")
    {  $empty = "y"; }
    my $form = $je->printForm(empty => $empty, input => $self->{input}, command => "saveCheck");
    if ($je->didErrorOccur)
    {
      $form = $je->errorMessage;
    }
    $doc->print($form);
  }

  return $doc;
}

sub c_saveCheck
{
  my $self = shift;
  my %args = ( message => "", @_ );
  my $phrases = $self->{langObj};
  my $doc = HTMLObject::Normal->new;
  if (not defined $doc || $doc->didErrorOccur)
  {
    $self->setError(errorString => "Could not create HTMLObject<br>\n");
    return undef;
  }
  $doc->setTitle($phrases->map("Save Check").":$self->{input}->{ledger}");
  $doc->setFocus("body");
  my $je = Portal::Accounting::Objects::HTMLJournalEntry->new(langObj => $self->{langObj});
  if ($je->didErrorOccur)
  {
    $doc->print("<h4>Error Occurred while initializing Journal Entry Object</h4>".$je->errorMessage."<br>\n");
    return $doc;
  }
  if ($je->populate(input => $self->{input}))
  { #put in db
    $doc->print("Writing jE to database<br>\n");
  }
  else
  {
    #redisplay
    my $form = $je->printForm();
    if ($je->didErrorOccur)
    {
      $form = "Journal Entry Error<br>\n";
      $form .= $je->errorMessage;
    }
    $doc->print("".$form);
  }

  return $doc;

}

=back

=cut

1;
__END__

=head1 NOTE

 All data fields are accessible by specifying the object
 and pointing to the data member to be modified on the
 left-hand side of the assignment.
 Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:xiwa@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3), Portal::AppState(3)

=cut
