# CGLManualEntry.pm - The Object Class that provides a CGLManualEntry Object
# Created by James Pattie 11/18/2003.
#$Id: CGLManualEntry.pm,v 1.2 2004/04/19 22:27:55 james Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::CGLManualEntry;
use strict;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();

$VERSION = '0.01';

# new
# instantiates an instance of a CGLManualEntry
# requires: name, number, gainLoss, type, refNum, sellDate
# optional: state
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = ( name => "", number => "", gainLoss => "0.00", state => "o", type => "SG", refNum => "", sellDate => "", @_ );
  my $errStr = "CGLManualEntry: new() - Error!<br>\n";

  $self->{name} = $args{name};
  $self->{number} = $args{number};
  $self->{gainLoss} = $args{gainLoss};
  $self->{state} = $args{state};
  $self->{type} = $args{type};
  $self->{refNum} = $args{refNum};
  $self->{sellDate} = $args{sellDate};

  # do validation
  $self->{error} = !$self->isValid;
  if ($self->{error})
  {
    $self->{errorString} = $errStr . $self->{errorString};
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";
  my $errStr = "CGLManualEntry: isValid() - Error!<br>\n";

  # do validation code here.
  if (length $self->{name} == 0)
  {
    $errorString .= "name must be specified!<br>\n";
  }
  # this is a subAccount and thus can be -1.
  if ($self->{number} !~ /^(-?\d+)$/)
  {
    $errorString .= "number = '$self->{number}' is invalid!<br>\n";
  }
  if ($self->{gainLoss} !~ /^(-?\d+\.\d{2})$/)
  {
    $errorString .= "gainLoss = '$self->{gainLoss}' is invalid!<br>\n";
  }
  if ($self->{state} !~ /^(o|c)$/)
  {
    $errorString .= "state = '$self->{state}' is invalid!<br>\n";
  }
  if ($self->{type} !~ /^(S|L)(G|L)$/)
  {
    $errorString .= "type = '$self->{type}' is invalid!<br>\n";
  }
  if ($self->{refNum} !~ /^(\d+)$/)
  {
    $errorString .= "refNum = '$self->{refNum}' is invalid!<br>\n";
  }
  if ($self->{sellDate} !~ /^(\d{2}[^\d]\d{2}[^\d]\d{4}|\d{4}[^\d]\d{2}[^\d]\d{2})$/)
  {
    $errorString .= "sellDate = '$self->{sellDate}' is invalid!<br>\n";
  }

  $error = (length $errorString > 0 ? 1 : 0);
  $self->{error} = $error;
  $self->{errorString} = $errStr if $error;
  $self->{errorString} .= $errorString;

  return !$error;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};
  
  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

1;
__END__

=head1 NAME

CGLManualEntry - Object used to build a CGLManualEntry Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::CGLManualEntry;
  my $obj = Portal::Accounting::Objects::CGLManualEntry->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

CGLManualEntry is a CGLManualEntry class.

=head1 Exported FUNCTIONS

  scalar new(name, number, gainLoss, state)
    Creates a new instance of the
    Portal::Accounting::Objects::CGLManualEntry object.

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will
    be available via errorMessage().

  void setError(errorString)
    Sets error = 1 and $self->{errorString} = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.


  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3)

=cut
