# CGLStockLotObject.pm - The Object Class that provides a CGLStockLotObject Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: CGLStockLotObject.pm,v 1.1.1.1 2001/08/09 19:04:50 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::CGLStockLotObject;
use strict;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();
	
$VERSION = '0.01';

# new 
# instantiates an instance of a CGLStockLotObject
# requires: purchased, sold, cost, market, gainLoss, shares, type, refNum
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = ( refNum => "", purchased => "", sold => "", cost => "0.00", market => "0.00", gainLoss => "0.00", shares => 0, type => "S", @_ );
  my $errStr = "CGLStockLotObject: new() - Error!<br>\n";

  $self->{refNum} = $args{refNum};
  $self->{cost} = $args{cost};
  $self->{market} = $args{market};
  $self->{gainLoss} = $args{gainLoss};
  $self->{purchased} = $args{purchased};
  $self->{sold} = $args{sold};
  $self->{shares} = $args{shares};
  $self->{type} = $args{type};
  
  # do validation
  $self->{error} = !$self->isValid;  
  if ($self->{error})
  {
    $self->{errorString} = $errStr . $self->{errorString};
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";
  my $errStr = "CGLStockLotObject: isValid() - Error!<br>\n";

  # do validation code here.
  if ($self->{refNum} !~ /^(\d+)$/)
  {
    $errorString .= "refNum = '$self->{refNum}' is invalid!<br>\n";
  }
  if ($self->{cost} !~ /^(-?\d+\.\d{2})$/)
  {
    $errorString .= "cost = '$self->{cost}' is invalid!<br>\n";
  }
  if ($self->{market} !~ /^(-?\d+\.\d{2})$/)
  {
    $errorString .= "market = '$self->{market}' is invalid!<br>\n";
  }
  if ($self->{gainLoss} !~ /^(-?\d+\.\d{2})$/)
  {
    $errorString .= "gainLoss = '$self->{gainLoss}' is invalid!<br>\n";
  }
  if ($self->{shares} !~ /^(\d+)$/)
  {
    $errorString .= "shares = '$self->{shares}' is invalid!<br>\n";
  }
  elsif ($self->{shares} <= 0)
  {
    $errorString .= "shares = '$self->{shares}' is invalid!<br>\n";
  }
  if ($self->{type} !~ /^(S|L)$/)
  {
    $errorString .= "type = '$self->{type}' is invalid!<br>\n";
  }
  if ($self->{purchased} !~ /^\d{2}(-|\/)\d{2}(-|\/)\d{4}$/)
  {
    $errorString .= "purchased = '$self->{purchased}' is invalid!<br>\n";
  }
  if ($self->{sold} !~ /^\d{2}(-|\/)\d{2}(-|\/)\d{4}$/)
  {
    $errorString .= "sold = '$self->{purchased}' is invalid!<br>\n";
  }

  $self->{error} = $error;
  $self->{errorString} = $errStr if $error;
  $self->{errorString} .= $errorString;

  return !$error;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};
  
  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

1;
__END__

=head1 NAME

CGLStockLotObject - Object used to build a CGLStockLotObject Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::CGLStockLotObject;
  my $obj = Portal::Accounting::Objects::CGLStockLotObject->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

CGLStockLotObject is a CGLStockLotObject class.

=head1 Exported FUNCTIONS

  scalar new(refNum, purchased, sold, cost, market, gainLoss, shares, 
             type)
    Creates a new instance of the
    Portal::Accounting::Objects::CGLStockLotObject object.

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will
    be available via errorMessage().

  void setError(errorString)
    Sets error = 1 and $self->{errorString} = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.


  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3)

=cut
