# CGLStocks.pm - The Object Class that provides a CGLStocks Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: CGLStocks.pm,v 1.1.1.1 2001/08/09 19:04:47 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::CGLStocks;
use strict;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();
	
$VERSION = '0.01';

# new 
# instantiates an instance of a CGLStocks
# requires: \@cglStockObjects, cost, market, gainLoss
# optional: state
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = ( cglStockObjects => undef, cost => "0.00", market => "0.00", gainLoss => "0.00", state => "o", @_ );
  my $errStr = "CGLStocks: new() - Error!<br>\n";

  $self->{cost} = $args{cost};
  $self->{market} = $args{market};
  $self->{gainLoss} = $args{gainLoss};
  $self->{cglStockObjects} = $args{cglStockObjects};
  $self->{state} = $args{state};
  
  # do validation
  $self->{error} = !$self->isValid;  
  if ($self->{error})
  {
    $self->{errorString} = $errStr . $self->{errorString};
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";
  my $errStr = "CGLStocks: isValid() - Error!<br>\n";

  # do validation code here.
  if ($self->{cost} !~ /^(-?\d+\.\d{2})$/)
  {
    $errorString .= "cost = '$self->{cost}' is invalid!<br>\n";
  }
  if ($self->{market} !~ /^(-?\d+\.\d{2})$/)
  {
    $errorString .= "market = '$self->{market}' is invalid!<br>\n";
  }
  if ($self->{gainLoss} !~ /^(-?\d+\.\d{2})$/)
  {
    $errorString .= "gainLoss = '$self->{gainLoss}' is invalid!<br>\n";
  }
  if (! defined $self->{cglStockObjects})
  {
    $errorString .= "cglStockObjects is not defined!<br>\n";
  }
  if ($self->{state} !~ /^(o|c)$/)
  {
    $errorString .= "state = '$self->{state}' is invalid!<br>\n";
  }

  $self->{error} = $error;
  $self->{errorString} = $errStr if $error;
  $self->{errorString} .= $errorString;

  return !$error;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};
  
  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

1;
__END__

=head1 NAME

CGLStocks - Object used to build a CGLStocks Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::CGLStocks;
  my $obj = Portal::Accounting::Objects::CGLStocks->new;
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

CGLStocks is a CGLStocks class.

=head1 Exported FUNCTIONS

  scalar new(cglStockObjects, cost, market, gainLoss, state)
    Creates a new instance of the
    Portal::Accounting::Objects::CGLStocks object.

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will
    be available via errorMessage().

  void setError(errorString)
    Sets error = 1 and $self->{errorString} = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.


  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3)

=cut
