# LotBalanceObject.pm - The Object Class that provides a Lot Balance Object
# that contains the data values from the table
# Created by Jason A. Pattie, 11/17/2000.
#$Id: LotBalanceObject.pm,v 1.1.1.1 2001/08/09 19:04:44 moreejt Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::LotBalanceObject;
use strict;
use vars qw($VERSION @ISA @EXPORT $AUTOLOAD);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();
	
$VERSION = '0.01';

# new 
# instantiates an instance of a LotBalanceObject
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my $errorHeader = "LotBalanceObject->new(): ";
  my %args = (ledgerCode => undef, subEntity => "", accountNum => undef, subAccountNum => undef, lot => undef,
              shares => undef, cost => undef, eventDate => undef, eventRefNum => undef, @_ );

  $self->{error} = 0;  # nothing wrong yet.
  $self->{errorString} = "";

  $self->{ledgerCode}     = $args{'ledgerCode'};
  $self->{subEntity}      = $args{'subEntity'};
  $self->{accountNum}     = $args{'accountNum'};
  $self->{subAccountNum}  = $args{'subAccountNum'};
  $self->{lot}            = $args{'lot'};
  $self->{shares}         = $args{'shares'};
  $self->{cost}           = $args{'cost'};
  $self->{eventDate}      = $args{'eventDate'};
  $self->{eventRefNum}    = $args{'eventRefNum'};

  if (not $self->isValid())
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Error initializing object!<br>\n" . $self->{errorString};
  }

  return $self;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# isValid
# validates the object in its current state
sub isValid
{
  my $self = shift;
  my $errorHeader = "LotBalanceObject->isValid(): ";

  if ((not defined $self->{ledgerCode}) or $self->{ledgerCode} eq "")
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Ledger code not defined or empty!";
    return 0;
  }
  if ((not defined $self->{accountNum}) or $self->{accountNum} eq "")
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Account number not defined or empty!";
    return 0;
  }
  if ($self->{accountNum} !~ /^[0-9]+$/)
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Account number does not appear to be numeric!";
    return 0;
  }
  if ((not defined $self->{subAccountNum}) or $self->{subAccountNum} eq "")
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Sub-account number not defined or empty!";
    return 0;
  }
  if ($self->{subAccountNum} !~ /^[0-9]+$/)
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Sub-account number does not appear to be numeric!";
    return 0;
  }
  if ((not defined $self->{lot}) or $self->{lot} eq "")
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Lot number not defined or empty!";
    return 0;
  }
  if ($self->{lot} !~ /^[0-9]+$/)
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Lot number does not appear to be numeric!";
    return 0;
  }
  if ((not defined $self->{shares}) or $self->{shares} eq "")
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Shares not defined or empty!";
    return 0;
  }
  if ($self->{shares} !~ /^-?(([0-9]+)|([0-9]*\.[0-9]+))$/)
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Shares does not appear to be numeric!";
    return 0;
  }
  if ($self->{shares} !~ /^(([0-9]+)|([0-9]*\.[0-9]+))$/)
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Shares cannot be negative!";
    return 0;
  }
  if ((not defined $self->{cost}) or $self->{cost} eq "")
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Cost not defined or empty!";
    return 0;
  }
  if ($self->{cost} !~ /^-?(([0-9]+)|([0-9]*\.[0-9]+))$/)
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Cost does not appear to be numeric!";
    return 0;
  }
  if ($self->{cost} !~ /^(([0-9]+)|([0-9]*\.[0-9]+))$/)
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Cost cannot be negative!";
    return 0;
  }
  if ((not defined $self->{eventDate}) or $self->{eventDate} eq "")
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Event date not defined or empty!";
    return 0;
  }
  if ((not defined $self->{eventRefNum}) or $self->{eventRefNum} eq "")
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Event reference number not defined or empty!";
    return 0;
  }
  if ($self->{eventRefNum} !~ /^-?[0-9]+$/)
  {
    $self->{error} = 1;
    $self->{errorString} = $errorHeader . "Event reference number does not appear to be numeric!";
    return 0;
  }

  return 1;
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};
  
  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

1;
__END__

=head1 NAME

LotBalanceObject - Object used to build a lot balance entry that can be used for database transactions.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::LotBalanceObject;
  my $lotbalobj = SecurityEventObject->new(ledgerCode => 'pxi',
                                           subEntity => 'lst',
                                           accountNum => 24503,
                                           subAccountNum => 1530,
                                           lot => 2222,
                                           shares => 75,
                                           cost => 295.75,
                                           eventDate => '12-13-1999',
                                           eventRefNum => 2222);
  if ($lotbalobj->didErrorOccur())
  {
    die $lotbalobj->errorMessage();
  }

  # set object data by specifying the data field desired
  $lotbalobj->accountNum(23540);
  $lotbalobj->lot(3322);

  # make sure the object is useful by calling the validation routine
  # before writing it to the database
  if (not $lotbalobj->isValid())
  {
    die $lotbalobj->errorMessage();
  }

=head1 DESCRIPTION

LotBalanceObject is a class that stores data for a lot balance
transaction.  It provides a container for a lot balance entry that will
be used to write to the database or update an existing record in the
database.

=head1 Exported FUNCTIONS

  scalar new(ledgerCode, subEntity, accountNum, subAccountNum, lot,
             shares, cost, eventDate, eventRefNum)
    Creates a new instance of the
    Portal::Accounting::Objects::LotBalanceObject
    object.

  void setError(errorString)
    Sets error = 1 and $self->{errorString} = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.

  scalar isValid(void)
    Runs through the data fields contained in the class to make sure
    they contain valid data.  This function should be called before
    using this object in a database transaction.

  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.

=head1 AUTHOR

Jason A. Pattie (mailto:pattieja@pcxperience.com)

=head1 SEE ALSO

perl(1)

=cut