# PortfolioStockObject.pm - The Object Class that provides a PortfolioStock Object
# Created by James Pattie & Jason A. Pattie, 11/07/2000.
#$Id: PortfolioStockObject.pm,v 1.2 2003/04/01 20:42:51 pcxuser Exp $
#This package is released under the GNU General Public License.
#You should have recieved a copy of the GPL with it.
# Copyright (c) 2000 HLR http://www.pcxperience.org  All rights reserved.

package Portal::Accounting::Objects::PortfolioStockObject;
use strict;
use vars qw($AUTOLOAD $VERSION @ISA @EXPORT);

require Exporter;

@ISA = qw(Exporter AutoLoader);
@EXPORT = qw();
	
$VERSION = '0.01';

# new 
# instantiates an instance of a PortfolioStockObject
# takes: subAccountName, subAccountNum, totalShares, totalCost, totalMarketValue, accountNum, totalGainLoss, status, lots, averageCostPerShare
sub new
{
  my $that = shift;
  my $class = ref($that) || $that;
  my $self = bless {}, $class;
  my %args = ( subAccountName => "", subAccountNum => 0, totalShares => 0, totalCost => "0.00", totalMarketValue => "0.00", 
               accountNum => 0, totalGainLoss => "0.00", status => "c", lots => undef, averageCostPerShare => "0.00", newPricePerShare => "0.00", @_ );
               
  $self->{subAccountName} = $args{subAccountName};
  $self->{subAccountNum} = $args{subAccountNum};
  $self->{totalShares} = $args{totalShares};
  $self->{totalCost} = $args{totalCost};
  $self->{totalMarketValue} = $args{totalMarketValue};
  $self->{accountNum} = $args{accountNum};
  $self->{totalGainLoss} = $args{totalGainLoss};
  $self->{status} = $args{status};
  $self->{lots} = $args{lots};
  $self->{averageCostPerShare} = $args{averageCostPerShare};
  $self->{newPricePerShare} = $args{newPricePerShare};

  # do validation
  $self->{error} = !$self->isValid;  
  if ($self->{error})
  {
    $self->{errorString} = "PortfolioStockObject: new() - Error!<br>\n" . $self->{errorString};
  }

  return $self;
}

# isValid - Returns 0 or 1 to indicate if the object is valid.
sub isValid
{
  my $self = shift;
  my $error = 0;
  my $errorString = "";

  # do validation code here.
  if ($self->{accountNum} < 0)
  {
    $error = 1;
    $errorString .= "accountNum = '$self->{accountNum}' is invalid!<br>\n";
  }
  if ($self->{subAccountNum} < 0)
  {
    $error = 1;
    $errorString .= "subAccountNum = '$self->{subAccountNum}' is invalid!<br>\n";
  }
  if (length $self->{subAccountName} == 0)
  {
    $error = 1;
    $errorString .= "subAccountName = '$self->{subAccountName}' is invalid!<br>\n";
  }
  if ($self->{status} !~ /^(o|c)$/)
  {
    $error = 1;
    $errorString .= "status = '$self->{status}' is invalid!<br>\n";
  }
  #if (scalar @{$self->{lots}} == 0)
  #{
  #  $error = 1;
  #  $errorString .= "lots must be defined!<br>\n";
  #}
  if ($self->{totalShares} < 0)
  {
    $error = 1;
    $errorString .= "totalShares = '$self->{totalShares}' is invalid!<br>\n";
  }
  if ($self->{totalCost} !~ /^-?\d+\.\d+$/)
  {
    $error = 1;
    $errorString .= "totalCost = '$self->{totalCost}' is invalid!<br>\n";
  }
  if ($self->{totalMarketValue} !~ /^-?\d+\.\d+$/)
  {
    $error = 1;
    $errorString .= "totalMarketValue = '$self->{totalMarketValue}' is invalid!<br>\n";
  }
  if ($self->{totalGainLoss} !~ /^-?\d+\.\d+$/)
  {
    $error = 1;
    $errorString .= "totalGainLoss = '$self->{totalGainLoss}' is invalid!<br>\n";
  }
  if ($self->{averageCostPerShare} !~ /^-?\d+\.\d+$/)
  {
    $error = 1;
    $errorString .= "averageCostPerShare = '$self->{averageCostPerShare}' is invalid!<br>\n";
  }
  if ($self->{newPricePerShare} !~ /^-?\d+\.\d+$/)
  {
    $error = 1;
    $errorString .= "newPricePerShare = '$self->{newPricePerShare}' is invalid!<br>\n";
  }

  $self->{error} = $error;
  $self->{errorString} = "PortfolioStockObject: isValid() - Error!<br>\n" if $error;
  $self->{errorString} .= $errorString;

  return !$error;
}

sub DESTROY
{
  my $self = shift;
}

sub AUTOLOAD
{
  my $self = shift;
  my $type = ref($self) || die "$self is not an object";
  my $name = $AUTOLOAD;
  $name =~ s/.*://;	# strip fully-qualified portion
  unless (exists $self->{$name})
  {
    die "Can't access `$name' field in object of class $type";
  }
  if (@_)
  {
    return $self->{$name} = shift;
  }
  else
  {
    return $self->{$name};
  }
}

# setError
# requires: errorString
# returns: nothing
sub setError
{
  my $self = shift;
  my %args = ( errorString => "", @_ );
  my $errorString = $args{errorString};
  
  $self->{errorString} = $errorString;
  $self->{error} = 1;
}

# didErrorOccur
# returns the value of error.
sub didErrorOccur
{
  my $self = shift;

  return $self->{error};
}

# errorMessage
# returns the value of errorString
sub errorMessage
{
  my $self = shift;

  return $self->{errorString};
}

1;
__END__

=head1 NAME

PortfolioStockObject - Object used to build a PortfolioStock Object Class.

=head1 SYNOPSIS

  use Portal::Accounting::Objects::PortfolioStockObject;
  my @lots = array of PortfolioLotObjects;
  my $obj = Portal::Accounting::Objects::PortfolioStockObject->new(
            subAccountName => "RHAT", subAccountNum => 1245,
            totalShares => "10.00", totalCost => "60.00",
            totalMarketValue => "60.00", accountNum => 1234,
            totalGainLoss => "0.00", status => "o",
            lots => \@lots, averageCostPerShare => "6.00");
            );
  if ($obj->didErrorOccur())
  {
    die $obj->errorMessage();
  }

=head1 DESCRIPTION

PortfolioStockObject is a PortfolioStock class.  This represents a 
Stock entry in the Portfolio View.  This will contain all the info
for a specified Investment Account / Stock pair and will have all
Lots for the stock.

=head1 Exported FUNCTIONS

  scalar new(subAccountName, subAccountNum, totalShares, totalCost,
             totalMarketValue, accountNum, totalGainLoss, status, lots,
             averageCostPerShare)
    Creates a new instance of the
    Portal::Accounting::Objects::PortfolioStockObject object.
    lots = array ref of PortfolioLotObjects.
    status = (o-open, c-closed).

  bool isValid(void)
    Returns 0 or 1 to indicate if the object is valid.  The error will
    be available via errorMessage().

  void setError(errorString)
    Sets error = 1 and $self->{errorString} = errorString.

  scalar didErrorOccur(void)
    Returns the value of error.

  scalar errorMessage(void)
    Returns the value of errorString.


  NOTE:  All data fields are accessible by specifying the object
         and pointing to the data member to be modified on the
         left-hand side of the assignment.
         Ex.  $obj->variable($newValue); or $value = $obj->variable;

=head1 AUTHOR

PC & Web Xperience, Inc. (mailto:admin@pcxperience.com)

=head1 SEE ALSO

perl(1), Portal(3), Portal::Accounting(3)

=cut
